@@ -11,24 +11,12 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
-
-(defglobal *core-pathname* nil
-  "The absolute pathname of the running SBCL core.")
-
-(defglobal *runtime-pathname* nil
-  "The absolute pathname of the running SBCL runtime.")
-
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defconstant max-hash sb!xc:most-positive-fixnum))
-
-(def!type hash ()
-  `(integer 0 ,max-hash))
+(in-package "SB-IMPL")
 
 ;;; A number that can represent an index into a vector, including
 ;;; one-past-the-end
 (deftype array-range ()
-  '(integer 0 #.sb!xc:array-dimension-limit))
+  `(integer 0 ,sb-xc:array-dimension-limit))
 
 ;;; a type used for indexing into sequences, and for related
 ;;; quantities like lengths of lists and other sequences.
@@ -47,33 +35,13 @@
 ;;; MOST-POSITIVE-FIXNUM lets the system know it can increment a value
 ;;; of type INDEX without having to worry about using a bignum to
 ;;; represent the result.
-(def!type index () `(integer 0 (,sb!xc:array-dimension-limit)))
-
-;;; like INDEX, but only up to half the maximum. Used by hash-table
-;;; code that does plenty to (aref v (* 2 i)) and (aref v (1+ (* 2 i))).
-(def!type index/2 () `(integer 0 (,(floor sb!xc:array-dimension-limit 2))))
+(def!type index () `(integer 0 (,sb-xc:array-dimension-limit)))
 
 ;;; like INDEX, but augmented with -1 (useful when using the index
 ;;; to count downwards to 0, e.g. LOOP FOR I FROM N DOWNTO 0, with
 ;;; an implementation which terminates the loop by testing for the
 ;;; index leaving the loop range)
-(def!type index-or-minus-1 () `(integer -1 (,sb!xc:array-dimension-limit)))
-
-;;; A couple of VM-related types that are currently used only on the
-;;; alpha and mips platforms. -- CSR, 2002-06-24
-(def!type unsigned-byte-with-a-bite-out (size bite)
-  (unless (typep size '(integer 1))
-    (error "Bad size for the ~S type specifier: ~S."
-           'unsigned-byte-with-a-bite-out size))
-  (let ((bound (ash 1 size)))
-    `(integer 0 ,(- bound bite 1))))
-
-(def!type signed-byte-with-a-bite-out (size bite)
-  (unless (typep size '(integer 2))
-    (error "Bad size for ~S type specifier: ~S."
-            'signed-byte-with-a-bite-out size))
-  (let ((bound (ash 1 (1- size))))
-    `(integer ,(- bound) ,(- bound bite 1))))
+(def!type index-or-minus-1 () `(integer -1 (,sb-xc:array-dimension-limit)))
 
 ;;; The smallest power of two that is equal to or greater than X.
 (declaim (inline power-of-two-ceiling))
@@ -81,62 +49,21 @@
   (declare (type index x))
   (ash 1 (integer-length (1- x))))
 
-(def!type load/store-index (scale lowtag min-offset
-                                 &optional (max-offset min-offset))
-  `(integer ,(- (truncate (+ (ash 1 16)
-                             (* min-offset sb!vm:n-word-bytes)
-                             (- lowtag))
-                          scale))
-            ,(truncate (- (+ (1- (ash 1 16)) lowtag)
-                          (* max-offset sb!vm:n-word-bytes))
-                       scale)))
-
 (declaim (inline align-up))
 (defun align-up (value granularity &aux (mask (1- granularity)))
   (logandc2 (+ value mask) mask))
 
-#!+(or x86 x86-64)
-(defun displacement-bounds (lowtag element-size data-offset)
-  (let* ((adjustment (- (* data-offset sb!vm:n-word-bytes) lowtag))
-         (bytes-per-element (ceiling element-size sb!vm:n-byte-bits))
-         (min (truncate (+ sb!vm::minimum-immediate-offset adjustment)
-                        bytes-per-element))
-         (max (truncate (+ sb!vm::maximum-immediate-offset adjustment)
-                        bytes-per-element)))
-    (values min max)))
-
-#!+(or x86 x86-64)
-(def!type constant-displacement (lowtag element-size data-offset)
-  (flet ((integerify (x)
-           (etypecase x
-             (integer x)
-             (symbol (symbol-value x)))))
-    (let ((lowtag (integerify lowtag))
-          (element-size (integerify element-size))
-          (data-offset (integerify data-offset)))
-      (multiple-value-bind (min max) (displacement-bounds lowtag
-                                                          element-size
-                                                          data-offset)
-        `(integer ,min ,max)))))
-
 ;;; CHAR-CODE values for ASCII characters which we care about but
 ;;; which aren't defined in section "2.1.3 Standard Characters" of the
-;;; ANSI specification for Lisp
-;;;
-;;; KLUDGE: These are typically used in the idiom (CODE-CHAR
-;;; FOO-CHAR-CODE). I suspect that the current implementation is
-;;; expanding this idiom into a full call to CODE-CHAR, which is an
-;;; annoying overhead. I should check whether this is happening, and
-;;; if so, perhaps implement a DEFTRANSFORM or something to stop it.
-;;; (or just find a nicer way of expressing characters portably?) --
-;;; WHN 19990713
+;;; ANSI specification for Lisp.
+;;; These are typically used in the idiom (CODE-CHAR FOO-CHAR-CODE)
 (defconstant bell-char-code 7)
 (defconstant backspace-char-code 8)
 (defconstant tab-char-code 9)
 (defconstant line-feed-char-code 10)
 (defconstant form-feed-char-code 12)
 (defconstant return-char-code 13)
-(defconstant escape-char-code 27)
+(defconstant escape-char-code 27) ; unused
 (defconstant rubout-char-code 127)
 
 ;;;; type-ish predicates
@@ -200,25 +127,6 @@
     (sequence
      (>= (length sequence) length))))
 
-;;; Is X is a positive prime integer?
-(defun positive-primep (x)
-  ;; This happens to be called only from one place in sbcl-0.7.0, and
-  ;; only for fixnums, we can limit it to fixnums for efficiency. (And
-  ;; if we didn't limit it to fixnums, we should use a cleverer
-  ;; algorithm, since this one scales pretty badly for huge X.)
-  (declare (fixnum x))
-  (if (<= x 5)
-      (and (>= x 2) (/= x 4))
-      (and (not (evenp x))
-           (not (zerop (rem x 3)))
-           (do ((q 6)
-                (r 1)
-                (inc 2 (logxor inc 6)) ;; 2,4,2,4...
-                (d 5 (+ d inc)))
-               ((or (= r 0) (> d q)) (/= r 0))
-             (declare (fixnum inc))
-             (multiple-value-setq (q r) (truncate x d))))))
-
 ;;; Could this object contain other objects? (This is important to
 ;;; the implementation of things like *PRINT-CIRCLE* and the dumper.)
 (defun compound-object-p (x)
@@ -339,21 +247,6 @@
     (when (and (eq (car pair) item) (not (null pair)))
       (return pair))))
 
-;;; like (DELETE .. :TEST #'EQ):
-;;;   Delete all LIST entries EQ to ITEM (destructively modifying
-;;;   LIST), and return the modified LIST.
-(defun delq (item list)
-  (declare (explicit-check))
-  (let ((list list))
-    (do ((x list (cdr x))
-         (splice '()))
-        ((endp x) list)
-      (cond ((eq item (car x))
-             (if (null splice)
-                 (setq list (cdr x))
-                 (rplacd splice (cdr x))))
-            (t (setq splice x)))))) ; Move splice along to include element.
-
 ;;; Delete just one item
 (defun delq1 (item list)
   (do ((prev nil x)
@@ -365,15 +258,6 @@
           (rplacd prev (cdr x)))
       (return list))))
 
-;;; like (POSITION .. :TEST #'EQ):
-;;;   Return the position of the first element EQ to ITEM.
-(defun posq (item list)
-  (do ((i list (cdr i))
-       (j 0 (1+ j)))
-      ((null i))
-    (when (eq (car i) item)
-      (return j))))
-
 (declaim (inline neq))
 (defun neq (x y)
   (not (eq x y)))
@@ -440,19 +324,6 @@
              (setq ,val (pop ,tail))
              (progn ,@body)))))
 
-;;; Like GETHASH if HASH-TABLE contains an entry for KEY.
-;;; Otherwise, evaluate DEFAULT, store the resulting value in
-;;; HASH-TABLE and return two values: 1) the result of evaluating
-;;; DEFAULT 2) NIL.
-(defmacro ensure-gethash (key hash-table &optional default)
-  (with-unique-names (n-key n-hash-table value foundp)
-    `(let ((,n-key ,key)
-           (,n-hash-table ,hash-table))
-       (multiple-value-bind (,value ,foundp) (gethash ,n-key ,n-hash-table)
-         (if ,foundp
-             (values ,value t)
-             (values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
-
 ;;; (binding* ({(names initial-value [flag])}*) body)
 ;;; FLAG may be NIL or :EXIT-IF-NULL
 ;;;
@@ -542,7 +413,7 @@ precise source locations in case conditions are signaled during the
 execution of BODY.
 
 NOTE: This interface is experimental and subject to change."
-  #-sb-xc-host `(sb!c::call-with-current-source-form
+  #-sb-xc-host `(sb-c::call-with-current-source-form
                  (lambda () ,@body) ,@forms)
   #+sb-xc-host `(progn (list ,@forms) ,@body))
 
@@ -601,7 +472,7 @@ NOTE: This interface is experimental and subject to change."
     ;; in VALUES-SPECIFIER-TYPE. It's because SET calls VALUES-SPECIFIER-TYPE.
     (macrolet ((set! (symbol value)
                  `(#+sb-xc-host set
-                   #-sb-xc-host sb!kernel:%set-symbol-global-value
+                   #-sb-xc-host sb-kernel:%set-symbol-global-value
                    ,symbol ,value))
                (reset-stats ()
                  ;; If statistics gathering is not not compiled-in,
@@ -610,7 +481,7 @@ NOTE: This interface is experimental and subject to change."
                  ;; it is inconsequential to performance.
                  (if *profile-hash-cache*
                      `(let ((statistics
-                             (let ((*package* (symbol-package symbol)))
+                             (let ((*package* (sb-xc:symbol-package symbol)))
                                (symbolicate symbol "STATISTICS"))))
                         (unless (boundp statistics)
                           (set! statistics
@@ -620,7 +491,7 @@ NOTE: This interface is experimental and subject to change."
       ;; global variable before the vector's header+length have been set.
       ;; Without a barrier, this would be theoretically possible if the
       ;; architecture allows out-of-order memory writes.
-      (sb!thread:barrier (:write)
+      (sb-thread:barrier (:write)
         (reset-stats)
         (setq cache (make-array size :initial-element 0)))
       (set! symbol cache))))
@@ -694,7 +565,7 @@ NOTE: This interface is experimental and subject to change."
          (line-type (let ((n (+ nargs nvalues)))
                       (if (<= n 3) 'cons `(simple-vector ,n))))
          (bind-hashval
-          `((,hashval (the (signed-byte #.sb!vm:n-fixnum-bits)
+          `((,hashval (the (signed-byte #.sb-vm:n-fixnum-bits)
                            (funcall ,hash-function ,@arg-vars)))
             (,cache ,var-name)))
          (probe-it
@@ -703,7 +574,7 @@ NOTE: This interface is experimental and subject to change."
                (let ((,hashval ,hashval) ; gets clobbered in probe loop
                      (,cache (truly-the ,cache-type ,cache)))
                  ;; FIXME: redundant?
-                 (declare (type (signed-byte #.sb!vm:n-fixnum-bits) ,hashval))
+                 (declare (type (signed-byte #.sb-vm:n-fixnum-bits) ,hashval))
                  (loop repeat 2
                     do (let ((,entry
                               (svref ,cache
@@ -712,12 +583,12 @@ NOTE: This interface is experimental and subject to change."
                            ;; This barrier is a no-op on all multi-threaded SBCL
                            ;; architectures. No CPU except Alpha will move a
                            ;; load prior to a load on which it depends.
-                           (sb!thread:barrier (:data-dependency))
+                           (sb-thread:barrier (:data-dependency))
                            (locally (declare (type ,line-type ,entry))
                              (let* ,(case (length temps)
                                      (2 `((,(first temps) (car ,entry))
                                           (,(second temps) (cdr ,entry))))
-                                     (3 (let ((arg-temp (sb!xc:gensym "ARGS")))
+                                     (3 (let ((arg-temp (sb-xc:gensym "ARGS")))
                                           `((,arg-temp (cdr ,entry))
                                             (,(first temps) (car ,entry))
                                             (,(second temps)
@@ -753,7 +624,7 @@ NOTE: This interface is experimental and subject to change."
                    ;; MUST NOT be observed by another thread before its cells
                    ;; are filled. Equally bad, the 'output' cells in the line
                    ;; could be 0 while the 'input' cells matched something.
-                   (sb!thread:barrier (:write))
+                   (sb-thread:barrier (:write))
                    (cond ((eql (svref ,cache idx1) 0)
                           (setf (svref ,cache idx1) ,entry))
                          ((eql (svref ,cache idx2) 0)
@@ -856,33 +727,6 @@ NOTE: This interface is experimental and subject to change."
           (t
            (return)))))
 
-;;;; package idioms
-
-;;; Note: Almost always you want to use FIND-UNDELETED-PACKAGE-OR-LOSE
-;;; instead of this function. (The distinction only actually matters when
-;;; PACKAGE-DESIGNATOR is actually a deleted package, and in that case
-;;; you generally do want to signal an error instead of proceeding.)
-(defun %find-package-or-lose (package-designator)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
-  (or (find-package package-designator)
-      (error 'simple-package-error
-             :package package-designator
-             :format-control "The name ~S does not designate any package."
-             :format-arguments (list package-designator))))
-
-;;; ANSI specifies (in the section for FIND-PACKAGE) that the
-;;; consequences of most operations on deleted packages are
-;;; unspecified. We try to signal errors in such cases.
-(defun find-undeleted-package-or-lose (package-designator)
-  #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
-  (let ((maybe-result (%find-package-or-lose package-designator)))
-    (if (package-%name maybe-result)    ; if not deleted
-        maybe-result
-        (error 'simple-package-error
-               :package maybe-result
-               :format-control "The package ~S has been deleted."
-               :format-arguments (list maybe-result)))))
-
 ;;;; various operations on names
 
 ;;; Is NAME a legal variable/function name?
@@ -894,6 +738,23 @@ NOTE: This interface is experimental and subject to change."
 (defun legal-fun-name-p (name)
   (values (valid-function-name-p name)))
 
+;;; * extended-function-designator: an object that denotes a function and that is one of:
+;;;   a function name (denoting the function it names in the global environment),
+;;;   or a function (denoting itself). The consequences are undefined if a function name
+;;;   is used as an extended function designator but it does not have a global definition
+;;;   as a function, or if it is a symbol that has a global definition as a macro
+;;;   or a special form.
+;;; Release 1.4.6 advertised that (disassemble 'a-macro) works, which entails a choice:
+;;; - if (EXTENDED-FUNCTION-DESIGNATOR-P) is to return NIL, then we have to
+;;;   add (OR (SATISFIES MACRO-FUNCTION) ...) to the signature of DISASSEMBLE.
+;;; - if (EXTENDED-FUNCTION-DESIGNATOR-P) is to return T, then we must avoid
+;;;   using this predicate in contexts that demand a function.
+(defun extended-function-designator-p (x)
+  (or (and (legal-fun-name-p x)
+           (fboundp x)
+           (not (and (symbolp x) (special-operator-p x))))
+      (functionp x)))
+
 (deftype function-name () '(satisfies legal-fun-name-p))
 
 ;;; Signal an error unless NAME is a legal function name.
@@ -926,7 +787,7 @@ NOTE: This interface is experimental and subject to change."
 
 (defun looks-like-name-of-special-var-p (x)
   (and (symbolp x)
-       (symbol-package x)
+       (sb-xc:symbol-package x)
        (let ((name (symbol-name x)))
          (and (> (length name) 2) ; to exclude '* and '**
               (char= #\* (aref name 0))
@@ -960,7 +821,7 @@ NOTE: This interface is experimental and subject to change."
           (let* ((name (first spec))
                  (exp-temp (gensym "ONCE-ONLY")))
             `(let ((,exp-temp ,(second spec))
-                   (,name (sb!xc:gensym ,(symbol-name name))))
+                   (,name (sb-xc:gensym ,(symbol-name name))))
                `(let ((,,name ,,exp-temp))
                   ,,(frob (rest specs) body))))))))
 
@@ -974,8 +835,6 @@ NOTE: This interface is experimental and subject to change."
 ;;; error indicating that a required &KEY argument was not supplied.
 ;;; This function is also useful for DEFSTRUCT slot defaults
 ;;; corresponding to required arguments.
-(declaim (ftype (function () #+(and sb-xc-host ccl) *
-                             #-(and sb-xc-host ccl) nil) missing-arg))
 (defun missing-arg ()
   (/show0 "entering MISSING-ARG")
   (error "A required &KEY or &OPTIONAL argument was not supplied."))
@@ -1015,20 +874,23 @@ NOTE: This interface is experimental and subject to change."
 ;;; The "more or less" bit is that the no-bound-at-all case is
 ;;; represented by NIL (not by * as in ANSI type specifiers); and in
 ;;; this case we return NIL.
+(declaim (ftype (sfunction (t) (or null real)) type-bound-number))
 (defun type-bound-number (x)
   (if (consp x)
       (destructuring-bind (result) x result)
       x))
 
-;;; some commonly-occurring CONSTANTLY forms
-(macrolet ((def-constantly-fun (name constant-expr)
-             `(progn
-                (declaim (ftype (sfunction * (eql ,constant-expr)) ,name))
-                (setf (symbol-function ',name)
-                      (constantly ,constant-expr)))))
-  (def-constantly-fun constantly-t t)
-  (def-constantly-fun constantly-nil nil)
-  (def-constantly-fun constantly-0 0))
+;;; Coerce a numeric type bound to the given type while handling
+;;; exclusive bounds.
+(defun coerce-numeric-bound (bound type)
+  (flet ((c (thing)
+           (case type
+             (rational (rational thing))
+             (t (coerce thing type)))))
+    (when bound
+      (if (consp bound)
+          (list (c (car bound)))
+          (c bound)))))
 
 
 ;;;; utilities for two-VALUES predicates
@@ -1186,7 +1048,7 @@ NOTE: This interface is experimental and subject to change."
 ;;; so just remove the decoration.
 #-sb-xc-host
 (eval-when (:compile-toplevel)
-  (sb!xc:defmacro sb-cold:preserving-host-function (form) form))
+  (sb-xc:defmacro sb-cold:preserving-host-function (form) form))
 
 (sb-cold:preserving-host-function
 (defun print-symbol-with-prefix (stream symbol &optional colon at)
@@ -1199,7 +1061,7 @@ NOTE: This interface is experimental and subject to change."
   (let ((*package* *keyword-package*))
     (write symbol :stream stream :escape t))))
 
-(declaim (special sb!pretty:*pprint-quote-with-syntactic-sugar*))
+(declaim (special sb-pretty:*pprint-quote-with-syntactic-sugar*))
 (sb-cold:preserving-host-function
 (defun print-type-specifier (stream type-specifier &optional colon at)
   (declare (ignore colon at))
@@ -1214,7 +1076,7 @@ NOTE: This interface is experimental and subject to change."
   ;; like CL:FUNCTION, CL:INTEGER, etc. to be printed without package
   ;; prefix but forces printing with package prefix for other
   ;; specifiers.
-  (let ((sb!pretty:*pprint-quote-with-syntactic-sugar* nil)
+  (let ((sb-pretty:*pprint-quote-with-syntactic-sugar* nil)
         (*package* *cl-package*))
     (prin1 type-specifier stream))))
 
@@ -1222,32 +1084,22 @@ NOTE: This interface is experimental and subject to change."
 (defun print-type (stream type &optional colon at)
   (print-type-specifier stream (type-specifier type) colon at)))
 
-(declaim (ftype (sfunction (index &key (:comma-interval (and (integer 1) index))) index)
-                decimal-with-grouped-digits-width))
-(defun decimal-with-grouped-digits-width (value &key (comma-interval 3))
-  (let ((digits (length (write-to-string value :base 10))))
-    (+ digits (floor (1- digits) comma-interval))))
-
 
 ;;;; Deprecating stuff
 
 (deftype deprecation-state ()
   '(member :early :late :final))
 
-(deftype deprecation-software-and-version ()
-  '(or string (cons string (cons string null))))
-
 (defun normalize-deprecation-since (since)
-  (unless (typep since 'deprecation-software-and-version)
-    (error 'simple-type-error
+  (typecase since
+    (string (values nil since))
+    ((cons string (cons string null)) (values (car since) (cadr since)))
+    (t (error 'simple-type-error
            :datum since
-           :expected-type 'deprecation-software-and-version
+           :expected-type '(or string (cons string (cons string null)))
            :format-control "~@<The value ~S does not designate a ~
                             version or a software name and a version.~@:>"
-           :format-arguments (list since)))
-  (if (typep since 'string)
-      (values nil since)
-      (values-list since)))
+           :format-arguments (list since)))))
 
 (defun normalize-deprecation-replacements (replacements)
   (if (or (not (listp replacements))
@@ -1314,38 +1166,6 @@ NOTE: This interface is experimental and subject to change."
        state (first since) (second since) namespace name replacements)
       (values state since replacements))))
 
-;;; For-effect-only variant of CHECK-DEPRECATED-THING for
-;;; type-specifiers that descends into compound type-specifiers.
-(defun %check-deprecated-type (type-specifier)
-  (let ((seen '()))
-    ;; KLUDGE: we have to use SPECIFIER-TYPE to sanely traverse
-    ;; TYPE-SPECIFIER and detect references to deprecated types. But
-    ;; then we may have to drop its cache to get the
-    ;; PARSE-DEPRECATED-TYPE condition when TYPE-SPECIFIER is parsed
-    ;; again later.
-    ;;
-    ;; Proper fix would be a
-    ;;
-    ;;   walk-type function type-specifier
-    ;;
-    ;; mechanism that could drive VALUES-SPECIFIER-TYPE but also
-    ;; things like this function.
-    (block nil
-      (handler-bind
-          ((sb!kernel::parse-deprecated-type
-             (lambda (condition)
-               (let ((type-specifier (sb!kernel::parse-deprecated-type-specifier
-                                      condition)))
-                 (aver (symbolp type-specifier))
-                 (unless (memq type-specifier seen)
-                   (push type-specifier seen)
-                   (check-deprecated-thing 'type type-specifier)))))
-           ((or error sb!kernel:parse-unknown-type)
-             (lambda (condition)
-               (declare (ignore condition))
-               (return))))
-        (specifier-type type-specifier)))))
-
 (defun check-deprecated-type (type-specifier)
   (typecase type-specifier
     ((or symbol cons)
@@ -1356,21 +1176,6 @@ NOTE: This interface is experimental and subject to change."
                   (eq type-specifier (find-class name nil)))
          (%check-deprecated-type name))))))
 
-;; This is the moral equivalent of a warning from /usr/bin/ld that
-;; "gets() is dangerous." You're informed by both the compiler and linker.
-(defun loader-deprecation-warn (stuff whence)
-  ;; Stuff is a list: ((<state> name . category) ...)
-  ;; For now we only deal with category = :FUNCTION so we ignore it.
-  (let ((warning-class
-         ;; We're only going to warn once (per toplevel form),
-         ;; so pick the most stern warning applicable.
-         (if (every (lambda (x) (eq (car x) :early)) stuff)
-             'simple-style-warning 'simple-warning)))
-    (warn warning-class
-          :format-control "Reference to deprecated function~P ~S~@[ from ~S~]"
-          :format-arguments
-          (list (length stuff) (mapcar #'second stuff) whence))))
-
 ;;; STATE is one of
 ;;;
 ;;;   :EARLY, for a compile-time style-warning.
@@ -1409,29 +1214,6 @@ NOTE: This interface is experimental and subject to change."
 ;;; - SB-C::STACK-ALLOCATE-VECTOR (policy)         since 1.0.19.7            -> Final: anytime
 ;;; - SB-C::STACK-ALLOCATE-VALUE-CELLS (policy)    since 1.0.19.7            -> Final: anytime
 
-(sb-cold:preserving-host-function
-(defun print-deprecation-replacements (stream replacements &optional colonp atp)
-  (declare (ignore colonp atp))
-  ;; I don't think this is callable during cross-compilation, is it?
-  ;; Anyway, the format string tokenizer can not handle APPLY on its own.
-  (apply #'format stream
-         (sb!format:tokens "~#[~;~
-             Use ~/sb-ext:print-symbol-with-prefix/ instead.~;~
-             Use ~/sb-ext:print-symbol-with-prefix/ or ~
-             ~/sb-ext:print-symbol-with-prefix/ instead.~:;~
-             Use~@{~#[~; or~] ~
-             ~/sb-ext:print-symbol-with-prefix/~^,~} instead.~
-           ~]")
-         replacements)))
-
-(defun print-deprecation-message (namespace name software version
-                                  &optional replacements stream)
-  (format stream
-           "The ~(~A~) ~/sb!ext:print-symbol-with-prefix/ has been ~
-            deprecated as of ~@[~A ~]version ~A.~
-            ~@[~2%~/sb!impl::print-deprecation-replacements/~]"
-          namespace name software version replacements))
-
 (defun setup-function-in-final-deprecation
     (software version name replacement-spec)
   #+sb-xc-host (declare (ignore software version name replacement-spec))
@@ -1446,7 +1228,7 @@ NOTE: This interface is experimental and subject to change."
 
 (defun setup-variable-in-final-deprecation
     (software version name replacement-spec)
-  (sb!c::%define-symbol-macro
+  (sb-c::%define-symbol-macro
    name
    `(deprecation-error
      ,software ,version 'variable ',name
@@ -1461,45 +1243,6 @@ NOTE: This interface is experimental and subject to change."
   (declare (ignore software version replacement-spec))
   (%compiler-deftype name (constant-type-expander name t) nil))
 
-(defmacro define-deprecated-function (state version name replacements lambda-list
-                                      &body body)
-  (declare (type deprecation-state state)
-           (type string version)
-           (type function-name name)
-           (type (or function-name list) replacements)
-           (type list lambda-list)
-           #+sb-xc-host (ignore version replacements))
-  `(progn
-     #-sb-xc-host
-     (declaim (deprecated
-               ,state ("SBCL" ,version)
-               (function ,name ,@(when replacements
-                                   `(:replacement ,replacements)))))
-     ,(ecase state
-        ((:early :late)
-         `(defun ,name ,lambda-list
-            ,@body))
-        ((:final)
-         `',name))))
-
-(defmacro define-deprecated-variable (state version name
-                                      &key (value nil valuep) replacement)
-  (declare (type deprecation-state state)
-           (type string version)
-           (type symbol name)
-           #+sb-xc-host (ignore version replacement))
-  `(progn
-     #-sb-xc-host
-     (declaim (deprecated
-               ,state ("SBCL" ,version)
-               (variable ,name ,@(when replacement
-                                   `(:replacement ,replacement)))))
-     ,(ecase state
-        ((:early :late)
-         `(defvar ,name ,@(when valuep (list value))))
-        ((:final)
-         `',name))))
-
 ;; Given DECLS as returned by from parse-body, and SYMBOLS to be bound
 ;; (with LET, MULTIPLE-VALUE-BIND, etc) return two sets of declarations:
 ;; those which pertain to the variables and those which don't.
@@ -1570,20 +1313,18 @@ NOTE: This interface is experimental and subject to change."
 (defmacro with-sane-io-syntax (&body forms)
   `(call-with-sane-io-syntax (lambda () ,@forms)))
 
-(!defvar *print-vector-length* nil
-  "Like *PRINT-LENGTH* but works on strings and bit-vectors.
-Does not affect the cases that are already controlled by *PRINT-LENGTH*")
-(declaim (always-bound *print-vector-length*))
-
 (defun call-with-sane-io-syntax (function)
   (declare (type function function))
   #-sb-xc-host (declare (dynamic-extent function)) ; "unable"
+  ;; force BOUNDP to be tested by declaring maximal safety
+  ;; in case unsafe code really screwed things up.
+  (declare (optimize (safety 3)))
   (macrolet ((true (sym)
                `(and (boundp ',sym) ,sym)))
     (let ((*print-readably* nil)
           (*print-level* (or (true *print-level*) 6))
           (*print-length* (or (true *print-length*) 12))
-          (*print-vector-length* (or (true *print-vector-length*) 200)))
+          #-sb-xc-host (*print-vector-length* (or (true *print-vector-length*) 200)))
       (funcall function))))
 
 ;;; Returns a list of members of LIST. Useful for dealing with circular lists.
@@ -1600,15 +1341,6 @@ Does not affect the cases that are already controlled by *PRINT-LENGTH*")
          (values members (or (not (listp tail))
                              (and (>= count max-length) :maybe)))))))
 
-;;; Default evaluator mode (interpeter / compiler)
-
-(declaim (type (member :compile #!+(or sb-eval sb-fasteval) :interpret)
-               *evaluator-mode*))
-(!defparameter *evaluator-mode* :compile
-  "Toggle between different evaluator implementations. If set to :COMPILE,
-an implementation of EVAL that calls the compiler will be used. If set
-to :INTERPRET, an interpreter will be used.")
-(declaim (always-bound *evaluator-mode*))
 ;; This is not my preferred name for this function, but chosen for harmony
 ;; with everything else that refers to these as 'hash-caches'.
 ;; Hashing is just one particular way of memoizing, and it would have been
@@ -1618,7 +1350,7 @@ to :INTERPRET, an interpreter will be used.")
 '(defun show-hash-cache-statistics ()
   (flet ((cache-stats (symbol)
            (let* ((name (string symbol))
-                  (statistics (let ((*package* (symbol-package symbol)))
+                  (statistics (let ((*package* (sb-xc:symbol-package symbol)))
                                 (symbolicate symbol "STATISTICS")))
                   (prefix
                    (subseq name 0 (- (length name) (length "VECTOR**")))))
@@ -1648,32 +1380,43 @@ to :INTERPRET, an interpreter will be used.")
                                 (length cache))))
                   short-name))))))
 
-(in-package "SB!KERNEL")
+;;; Just like WITH-OUTPUT-TO-STRING but doesn't close the stream,
+;;; producing more compact code.
+(defmacro with-simple-output-to-string
+    ((var &optional string (element-type :default)) &body body)
+  ;; Don't need any fancy type-specifier parsing. Uses of this macro
+  ;; are confined to our own code. So the element-type is literally
+  ;; either :DEFAULT or BASE-CHAR.
+  (aver (member element-type '(base-char :default)))
+  (multiple-value-bind (forms decls) (parse-body body nil)
+    (if string
+        `(let ((,var (make-fill-pointer-output-stream ,string)))
+           ,@decls
+           ,@forms)
+        ;; Why not dxify this stream?
+        `(let ((,var #+sb-xc-host (make-string-output-stream)
+                     #-sb-xc-host
+                     ,(case element-type
+                        ;; Non-unicode doesn't have %MAKE-DEFAULT-STRING-OSTREAM
+                        #+sb-unicode
+                        (:default '(%make-default-string-ostream))
+                        ;; and this macro is never employed to make streams
+                        ;; that can only return CHARACTER string.
+                        (t '(%make-base-string-ostream)))))
+           ,@decls
+           ,@forms
+           (get-output-stream-string ,var)))))
+
+(in-package "SB-KERNEL")
 
 (defun fp-zero-p (x)
   (typecase x
     (single-float (zerop x))
     (double-float (zerop x))
-    #!+long-float
+    #+long-float
     (long-float (zerop x))
     (t nil)))
 
-(defun neg-fp-zero (x)
-  (etypecase x
-    (single-float
-     (if (eql x 0.0f0)
-         (make-unportable-float :single-float-negative-zero)
-         0.0f0))
-    (double-float
-     (if (eql x 0.0d0)
-         (make-unportable-float :double-float-negative-zero)
-         0.0d0))
-    #!+long-float
-    (long-float
-     (if (eql x 0.0l0)
-         (make-unportable-float :long-float-negative-zero)
-         0.0l0))))
-
 (declaim (inline schwartzian-stable-sort-list))
 (defun schwartzian-stable-sort-list (list comparator &key key)
   (if (null key)
@@ -1700,38 +1443,30 @@ to :INTERPRET, an interpreter will be used.")
              (sorted (stable-sort wrapped comparator :key #'cdr)))
         (map-into sorted #'car sorted))))
 
-;;; Just like WITH-OUTPUT-TO-STRING but doesn't close the stream,
-;;; producing more compact code.
-(defmacro with-simple-output-to-string
-    ((var &optional string)
-     &body body)
-  (multiple-value-bind (forms decls) (parse-body body nil)
-    (if string
-        `(let ((,var (sb!impl::make-fill-pointer-output-stream ,string)))
-           ,@decls
-           ,@forms)
-        `(let ((,var #+sb-xc-host (make-string-output-stream)
-                     #-sb-xc-host (sb!impl::%make-string-output-stream
-                                   (or #!-sb-unicode 'character :default)
-                                   #'sb!impl::string-ouch)))
-
-           ,@decls
-           ,@forms
-           (get-output-stream-string ,var)))))
-
 ;;; Ensure basicness if possible, and simplicity always
 (defun possibly-base-stringize (s)
   (declare (string s))
-  (cond #!+(and sb-unicode (host-feature sb-xc))
+  (cond #+(and sb-unicode (not sb-xc-host))
         ((and (typep s '(array character (*))) (every #'base-char-p s))
          (coerce s 'base-string))
         (t
          (coerce s 'simple-string))))
 
+;;; Return T if X is an object that always evaluates to itself. Guard against:
+;;;  (LET ((S 'XXX)) (SET S 9) (UNINTERN S) (IMPORT S 'KEYWORD) (SYMBOL-VALUE S)) => 9
+;;; whereby :XXX satisfies KEWORDP and has value 9, or maybe even has no value
+;;; if not assigned anything previously.
+;;; Interestingly, CLISP and CCL cause the symbol-value to become itself
+;;; (and MAKUNBOUND to be illegal). That's an interesting choice which closes
+;;; a weird loophole, but is not universal in all implementations.
 (defun self-evaluating-p (x)
   (typecase x
     (null t)
-    (symbol (or (eq x t) (eq (symbol-package x) *keyword-package*)))
+    (symbol
+     (or (eq x t)
+         (and (eq (cl:symbol-package x) *keyword-package*)
+              (boundp x)
+              (eq (symbol-value x) x))))
     (cons nil)
     (t t)))
 
