@@ -19,12 +19,12 @@
 (defstruct (room-info (:constructor make-room-info (mask name kind))
                       (:copier nil))
     ;; the mask applied to HeaderValue to compute object size
-    (mask 0 :type (and fixnum unsigned-byte))
+    (mask 0 :type (and fixnum unsigned-byte) :read-only t)
     ;; the name of this type
     (name nil :type symbol :read-only t)
     ;; kind of type (how to reconstitute an object)
     (kind nil
-          :type (member :other :closure :instance :list :code :vector-nil)
+          :type (member :other :closure :instance :list :code :fdefn)
           :read-only t))
 
 (defun room-info-type-name (info)
@@ -43,17 +43,17 @@
         (when (and (eq lowtag 'other-pointer-lowtag)
                    (not (member widetag '(t nil))))
           (setf (svref infos (symbol-value widetag))
-                (make-room-info (if (member name '(fdefn symbol))
-                                    tiny-boxed-size-mask
-                                    default-size-mask)
-                                name :other)))))
+                (case name
+                 (fdefn  (make-room-info 0 name :fdefn))
+                 (symbol (make-room-info tiny-boxed-size-mask name :other))
+                 (t      (make-room-info default-size-mask name :other)))))))
 
-    (dolist (code (list #+sb-unicode complex-character-string-widetag
-                        complex-base-string-widetag simple-array-widetag
-                        complex-bit-vector-widetag complex-vector-widetag
-                        complex-array-widetag complex-vector-nil-widetag))
-      (setf (svref infos code)
-            (make-room-info default-size-mask 'array-header :other)))
+    (let ((info (make-room-info default-size-mask 'array-header :other)))
+      (dolist (code (list #+sb-unicode complex-character-string-widetag
+                          complex-base-string-widetag simple-array-widetag
+                          complex-bit-vector-widetag complex-vector-widetag
+                          complex-array-widetag complex-vector-nil-widetag))
+        (setf (svref infos code) info)))
 
     (setf (svref infos bignum-widetag)
           ;; Lose 1 more bit than n-widetag-bits because fullcgc robs 1 bit,
@@ -70,7 +70,7 @@
           (setf (svref infos (saetp-typecode saetp)) saetp))))
 
     (setf (svref infos simple-array-nil-widetag)
-          (make-room-info 0 'simple-array-nil :vector-nil))
+          (make-room-info 0 'simple-array-nil :other))
 
     (setf (svref infos code-header-widetag)
           (make-room-info 0 'code :code))
@@ -194,10 +194,6 @@
             (round-to-dualword (+ (* vector-data-offset n-word-bytes)
                                   n-data-octets)))))
 
-(defun code-component-size (x) ; in bytes
-  (declare (code-component x))
-  (round-to-dualword (+ (* (code-header-words x) n-word-bytes) (%code-code-size x))))
-
 (defun primitive-object-size (object)
   "Return number of bytes of heap or stack directly consumed by OBJECT"
   (if (is-lisp-pointer (get-lisp-obj-address object))
@@ -214,29 +210,25 @@
                 ;; why these fudge factors are right, but they make the result
                 ;; equal to what MAP-ALLOCATED-OBJECTS reports.
                 (null (+ symbol-size 1 #+64-bit 1))
+                (code-component
+                 (return-from primitive-object-size (code-object-size object)))
+                (fdefn 4) ; no length stored in the header
                 ;; Anything else is an OTHER pointer.
-                ;; Use a sizing function when we have one,
-                ;; otherwise the general case is correct.
                 (t
                  (let ((room-info
                          (aref *room-info* (%other-pointer-widetag object))))
-                   (typecase object
-                     (array
-                      (cond ((array-header-p object)
-                             (+ array-dimensions-offset (array-rank object)))
-                            ((simple-array-nil-p object) 2)
-                            (t
-                             (return-from primitive-object-size
-                               (nth-value 2 (reconstitute-vector
-                                             object room-info))))))
-                     (code-component
-                      (return-from primitive-object-size (code-component-size object)))
-                     (t
-                      ;; Other things (symbol, fdefn, value-cell, etc)
-                      ;; don't have a sizer, so use GET-HEADER-DATA
-                      (1+ (logand (get-header-data object)
-                                  (logand (get-header-data object)
-                                          (room-info-mask room-info)))))))))))
+                   (if (arrayp object)
+                       (cond ((array-header-p object)
+                              (+ array-dimensions-offset (array-rank object)))
+                             ((simple-array-nil-p object) 2)
+                             (t
+                              (return-from primitive-object-size
+                                (nth-value 2 (reconstitute-vector object room-info)))))
+                       ;; Other things (symbol, value-cell, etc)
+                       ;; don't have a sizer, so use GET-HEADER-DATA
+                       (1+ (logand (get-header-data object)
+                                   (logand (get-header-data object)
+                                           (room-info-mask room-info))))))))))
         (* (align-up words 2) n-word-bytes))
       0))
 
@@ -276,31 +268,30 @@
                    list-pointer-lowtag
                    (* 2 n-word-bytes)))
 
-          (:closure ; also funcallable-instance
-           (values (tagged-object fun-pointer-lowtag)
-                   widetag
-                   (boxed-size (logand header-value short-header-max-words))))
-
           (:instance
            (values (tagged-object instance-pointer-lowtag)
                    widetag
                    (boxed-size (logand header-value short-header-max-words))))
 
-          (:other
-           (values (tagged-object other-pointer-lowtag)
+          (:closure ; also funcallable-instance
+           (values (tagged-object fun-pointer-lowtag)
                    widetag
-                   (boxed-size (logand header-value (room-info-mask info)))))
-
-          (:vector-nil
-           (values (tagged-object other-pointer-lowtag)
-                   simple-array-nil-widetag
-                   (* 2 n-word-bytes)))
+                   (boxed-size (logand header-value short-header-max-words))))
 
           (:code
            (let ((c (tagged-object other-pointer-lowtag)))
              (values c
                      code-header-widetag
-                     (code-component-size c))))))))))
+                     (code-object-size c))))
+
+          (:fdefn
+           (values (tagged-object other-pointer-lowtag) widetag
+                   (* fdefn-size n-word-bytes)))
+
+          (:other
+           (values (tagged-object other-pointer-lowtag)
+                   widetag
+                   (boxed-size (logand header-value (room-info-mask info)))))))))))
 
 ;;; Iterate over all the objects in the contiguous block of memory
 ;;; with the low address at START and the high address just before
@@ -467,7 +458,7 @@ We could try a few things to mitigate this:
        ;; in fixedobj subspace, or code without enough header words
        ;; in varyobj subspace. (cf 'filler_obj_p' in gc-internal.h)
        (dx-flet ((filter (obj type size)
-                   (unless (consp obj)
+                   (unless (= type list-pointer-lowtag)
                      (funcall fun obj type size))))
          (map-immobile-objects #'filter :fixed))
        (dx-flet ((filter (obj type size)
@@ -545,7 +536,31 @@ We could try a few things to mitigate this:
                                     ,extra))))))
                  (map-objects-in-range fun start (range-end page-bytes-used)
                                        (< page-index initial-next-free-page))
-                 (setf start (range-end gencgc-card-bytes) span 0)))))))))
+                 ;; In a certain rare situation, we must restart the next loop iteration
+                 ;; at exactly PAGE-INDEX instead of 1+ PAGE-INDEX.
+                 ;; Normally the contents of PAGE-INDEX are always included in the
+                 ;; current range. But what if it contributed 0 bytes to the range?
+                 ;;
+                 ;;     N : #x8000 bytes used
+                 ;;   N+1 : #x8000 bytes used
+                 ;;   N+2 : 0 bytes used        <- PAGE-INDEX now points here
+                 ;;   N+3 : initially 0 bytes, then gets some bytes
+                 ;;
+                 ;; We invoked the map function with page_address(N) for #x10000 bytes.
+                 ;; Suppose that function consed a partly unboxed object starting on
+                 ;; page N+2 extending to page N+3, and that NEXT-FREE-PAGE was greater
+                 ;; than both to begin with, so the termination condition isn't in play.
+                 ;; In the best case scenario, resuming the scan at page N+3 (mid-object)
+                 ;; would read valid widetags, but in the worst case scenario, we get
+                 ;; random bits. Page N+2 needs a fighting chance to be the start of
+                 ;; a range, so go back 1 page if the current page had zero bytes used
+                 ;; and the span exceeded 1. That is, always make forward progress.
+                 (setq start (range-end (cond ((and (zerop page-bytes-used) (plusp span))
+                                               (decf page-index)
+                                               0)
+                                              (t
+                                               gencgc-card-bytes)))
+                       span 0)))))))))
 
   (do-rest-arg ((space) spaces)
     (if (eq space :dynamic)
@@ -585,10 +600,9 @@ We could try a few things to mitigate this:
              (hole-bytes 0))
     (map-immobile-objects
      (lambda (obj type size)
-       (declare (ignore type))
        (let ((address (logandc2 (get-lisp-obj-address obj) lowtag-mask)))
          (when (case subspace
-                 (:fixed (consp obj))
+                 (:fixed (= type list-pointer-lowtag))
                  (:variable (hole-p address)))
            (push (cons address size) holes)
            (incf hole-bytes size))))
@@ -622,8 +636,8 @@ We could try a few things to mitigate this:
 ;;; allocated in Space.
 (defun type-breakdown (space)
   (declare (muffle-conditions compiler-note))
-  (let ((sizes (make-array 256 :initial-element 0 :element-type '(unsigned-byte #.n-word-bits)))
-        (counts (make-array 256 :initial-element 0 :element-type '(unsigned-byte #.n-word-bits))))
+  (let ((sizes (make-array 256 :initial-element 0 :element-type 'word))
+        (counts (make-array 256 :initial-element 0 :element-type 'word)))
     (map-allocated-objects
      (lambda (obj type size)
        (declare (word size) (optimize (speed 3)) (ignore obj))
@@ -698,6 +712,12 @@ We could try a few things to mitigate this:
         (format t "~%Summary total:~%    ~:D bytes, ~:D objects.~%"
                 summary-total-bytes summary-total-objects)))))
 
+(declaim (ftype (sfunction (index &key (:comma-interval (and (integer 1) index))) index)
+                decimal-with-grouped-digits-width))
+(defun decimal-with-grouped-digits-width (value &key (comma-interval 3))
+  (let ((digits (length (write-to-string value :base 10))))
+    (+ digits (floor (1- digits) comma-interval))))
+
 ;;; Report object usage for a single space.
 (defun report-space-total (space-info cutoff)
   (declare (list space-info) (type (or single-float null) cutoff))
@@ -790,23 +810,35 @@ We could try a few things to mitigate this:
            (objects-width (decimal-with-grouped-digits-width total-objects))
            (totals-label (format nil "~:(~A~) instance total" space))
            (types-width (reduce #'max interesting
-                                :key (lambda (x)
-                                       (length (symbol-name (classoid-name (first x)))))
+                                :key (lambda (info)
+                                       (let ((type (first info)))
+                                         (length
+                                          (typecase type
+                                            (string
+                                             type)
+                                            (classoid
+                                             (with-output-to-string (stream)
+                                               (sb-ext:print-symbol-with-prefix
+                                                stream (classoid-name type))))))))
                                 :initial-value (length totals-label)))
            (printed-bytes 0)
            (printed-objects 0))
       (declare (unsigned-byte printed-bytes printed-objects))
       (flet ((type-usage (type objects bytes)
-               (let ((name (etypecase type
-                             (string type)
-                             (classoid (symbol-name (classoid-name type))))))
-                 (format t "  ~V@<~A~> ~V:D bytes, ~V:D object~:P.~%"
-                         (1+ types-width) name bytes-width bytes
-                         objects-width objects))))
-        (loop for (type . (objects . bytes)) in interesting do
-             (incf printed-bytes bytes)
-             (incf printed-objects objects)
-             (type-usage type objects bytes))
+               (etypecase type
+                 (string
+                  (format t "  ~V@<~A~> ~V:D bytes, ~V:D object~:P.~%"
+                          (1+ types-width) type bytes-width bytes
+                          objects-width objects))
+                 (classoid
+                  (format t "  ~V@<~/sb-ext:print-symbol-with-prefix/~> ~
+                             ~V:D bytes, ~V:D object~:P.~%"
+                          (1+ types-width) (classoid-name type) bytes-width bytes
+                          objects-width objects)))))
+        (loop for (type . (objects . bytes)) in interesting
+              do (incf printed-bytes bytes)
+                 (incf printed-objects objects)
+                 (type-usage type objects bytes))
         (terpri)
         (let ((residual-objects (- total-objects printed-objects))
               (residual-bytes (- total-bytes printed-bytes)))
@@ -822,6 +854,9 @@ We could try a few things to mitigate this:
 ;;; (2) sb-sprof deciding how many regions [sic] were made if #+cheneygc
 (defun get-page-size () sb-c:+backend-page-bytes+)
 
+;;; This function is sheer madness.  You're better off using
+;;; LIST-ALLOCATED-OBJECTS and then iterating over that, to avoid
+;;; seeing all the junk created while doing this thing.
 (defun print-allocated-objects (space &key (percent 0) (pages 5)
                                       type larger smaller count
                                       (stream *standard-output*))
@@ -918,24 +953,59 @@ We could try a few things to mitigate this:
 (defun list-allocated-objects (space &key type larger smaller count
                                      test)
   (declare (type (or (eql :all) spaces) space)
-           (type (or index null) larger smaller type count)
+           (type (or (unsigned-byte 8) null) type)
+           (type (or index null) larger smaller count)
            (type (or function null) test))
   (declare (dynamic-extent test))
-  (unless *ignore-after*
-    (setq *ignore-after* (cons 1 2)))
-  (collect ((counted 0 1+))
-    (let ((res ()))
-      (map-allocated-objects
-       (lambda (obj obj-type size)
-         (when (and (or (not type) (eql obj-type type))
-                    (or (not smaller) (<= size smaller))
-                    (or (not larger) (>= size larger))
-                    (or (not test) (funcall test obj)))
-           (setq res (maybe-cons space obj res))
-           (when (and count (>= (counted) count))
-             (return-from list-allocated-objects res))))
-       space)
-      res)))
+  (when (eql count 0)
+    (return-from list-allocated-objects nil))
+  ;; This function was pretty much random as to what subset of the heap it
+  ;; visited- it might see half the heap, 1/10th of the heap, who knows, because
+  ;; it stopped based on hitting a sentinel cons cell made just prior to the loop.
+  ;; That stopping condition was totally wrong because allocation does not occur
+  ;; linearly.  Taking 2 passes (first count, then store) stands a chance of
+  ;; getting a reasonable point-in-time view as long as other threads are not consing
+  ;; like crazy. If the user-supplied TEST function conses at all, then the result is
+  ;; still very arbitrary - including possible duplication of objects if we visit
+  ;; something and then see it again after GC transports it higher. The only way to
+  ;; allow consing in the predicate would be to use dedicated "arenas" for new
+  ;; allocations, that being a concept which we do not now - and may never - support.
+  (sb-int:dx-flet ((wantp (obj widetag size)
+                     (and (or (not type) (eql widetag type))
+                          (or (not smaller) (<= size smaller))
+                          (or (not larger) (>= size larger))
+                          (or (not test) (funcall test obj)))))
+    ;; Unless COUNT is smallish, always start by counting. Don't just trust the user
+    ;; because s/he might specify :COUNT huge-num which is acceptable provided that
+    ;; huge-num is an INDEX which could either exhaust the heap, or at least be
+    ;; wasteful if but a tiny handful of objects would actually satisfy WANTP.
+    (let* ((output (make-array
+                    (if (typep count '(integer 0 100000))
+                        count
+                        (let ((n 0))
+                          (map-allocated-objects
+                           (lambda (obj widetag size)
+                             (when (wantp obj widetag size) (incf n)))
+                           space)
+                          n))))
+           (index 0))
+      (block done
+       (map-allocated-objects
+        (lambda (obj widetag size)
+          (when (wantp obj widetag size)
+            (setf (aref output index) obj)
+            (when (= (incf index) (length output))
+              (return-from done))))
+        space))
+      (let ((list
+             (cond ((= index (length output)) ; easy case
+                    (coerce output 'list))
+                   (t ; didn't fill the array
+                    (collect ((res))
+                      (dotimes (i index (res))
+                        (res (svref output i))))))))
+        (fill output 0) ; assist GC a bit
+        list))))
 
 ;;; Calls FUNCTION with all objects that have (possibly conservative)
 ;;; references to them on current stack.
@@ -1059,23 +1129,28 @@ We could try a few things to mitigate this:
             ,.(make-case 'fdefn
                `(fdefn-name ,obj)
                `(fdefn-fun ,obj)
+               ;; While it looks like we could easily allow a pointer to a movable object
+               ;; in the fdefn-raw-addr slot, it is not exactly trivial- at a bare minimum,
+               ;; translating the raw-addr to a lispobj might have to be pseudoatomic,
+               ;; since we don't know what object to pin when reconstructing it.
+               ;; For simple-funs in dynamic space, it doesn't have to be pseudoatomic
+               ;; because a reference to the interior of code pins the code.
+               ;; Closure trampolines would be fine as well. That leaves funcallable instances
+               ;; as the pain point. Those could go on pages of code as well, but see the
+               ;; comment in conservative_root_p() in gencgc as to why that alone
+               ;; would be inadequate- we require a properly tagged descriptor
+               ;; to enliven any object other than code.
                #+immobile-code
                `(%make-lisp-obj
                  (alien-funcall (extern-alien "fdefn_callee_lispobj" (function unsigned unsigned))
                                 (logandc2 (get-lisp-obj-address ,obj) lowtag-mask))))
             ,.(make-case* 'code-component
-               `(,functoid (%code-debug-info ,obj) ,@more)
-               #+(or x86 immobile-code) `(,functoid (%code-fixups ,obj) ,@more)
-               `(loop for .i. from code-constants-offset below (code-header-words ,obj)
-                      do (,functoid (code-header-ref ,obj .i.) ,@more))
-               ;; Caller should extend behavior for embedded objects, like:
-               ;; `(loop for .i. below (code-n-entries ,obj)
-               ;;        do (,functoid (%code-entry-point ,obj .i.) ,@more)))
-               ;; and/or visit the slots of each simple-fun but not the fun per se.
-               )
+               `(loop for .i. from 2 below (code-header-words ,obj)
+                      do (,functoid (code-header-ref ,obj .i.) ,@more)))
             ,.(make-case '(or float (complex float) bignum
-                              #+sb-simd-pack simd-pack
-                              system-area-pointer)) ; nothing to do
+                           #+sb-simd-pack simd-pack
+                           #+sb-simd-pack-256 simd-pack-256
+                           system-area-pointer)) ; nothing to do
             ,.(make-case 'weak-pointer `(weak-pointer-value ,obj))
             ,.(make-case 'ratio `(%numerator ,obj) `(%denominator ,obj))
             ;; Visitor won't be invoked on (COMPLEX float)
@@ -1096,11 +1171,7 @@ We could try a few things to mitigate this:
           :extend
           (dotimes (i (code-n-entries this))
             (let ((f (%code-entry-point this i)))
-              (when (or (eq f that)
-                        (eq (%simple-fun-name f) that)
-                        (eq (%simple-fun-arglist f) that)
-                        (eq (%%simple-fun-type f) that)
-                        (eq (%simple-fun-info f) that))
+              (when (eq f that)
                 (go win)))))
          (t
           :extend
@@ -1203,7 +1274,7 @@ We could try a few things to mitigate this:
   (values))
 
 #+nil ; for debugging
-(defun dump-dynamic-space-code (&optional (stream *standard-output*)
+(defun show-dynamic-space-code (&optional (stream *standard-output*)
                                 &aux (n-code-bytes 0)
                                      (total-pages next-free-page)
                                      (pages
@@ -1247,6 +1318,23 @@ We could try a few things to mitigate this:
               n-code-bytes n-pages waste
               (* 100 (/ waste tot))))))
 
+#+nil ; for debugging
+(defun show-immobile-spaces (which)
+  (flet ((show (obj type size)
+           (declare (ignore type size))
+           (let ((*print-pretty* nil))
+             (format t "~x: ~s~%" (get-lisp-obj-address obj) obj))))
+    (when (or (eq which :fixed) (eq which :both))
+      (format t "Fixedobj space~%==============~%")
+      (map-objects-in-range #'show
+        (%make-lisp-obj fixedobj-space-start)
+        (%make-lisp-obj (sap-int *fixedobj-space-free-pointer*))))
+    (when (or (eq which :variable) (eq which :both))
+      (format t "Varyobj space~%=============~%")
+      (map-objects-in-range #'show
+        (%make-lisp-obj varyobj-space-start)
+        (%make-lisp-obj (sap-int *varyobj-space-free-pointer*))))))
+
 #+gencgc
 (defun generation-of (object)
   (let* ((addr (get-lisp-obj-address object))
@@ -1257,7 +1345,22 @@ We could try a few things to mitigate this:
            ;; SIMPLE-FUNs don't contain a generation byte
            (when (simple-fun-p object)
              (setq addr (get-lisp-obj-address (fun-code-header object))))
-           (logand #xF (sap-ref-8 (int-sap (logandc2 addr lowtag-mask)) 3))))))
+           (let ((sap (int-sap (logandc2 addr lowtag-mask))))
+             (logand (if (fdefn-p object) (sap-ref-8 sap 1) (sap-ref-8 sap 3))
+                     #xF))))))
+
+;;; Show objects in a much simpler way than print-allocated-objects.
+;;; Probably don't use this for generation 0 of dynamic space. Other spaces are ok.
+;;; (And this is removed from the image; it's meant for developers)
+#+gencgc
+(defun show-generation-objs (gen space)
+  (let ((*print-pretty* nil))
+    (map-allocated-objects
+     (lambda (obj type size)
+       (declare (ignore type))
+       (when (= (generation-of obj) gen)
+         (format t "~x ~3x ~s~%" (get-lisp-obj-address obj) size obj)))
+     space)))
 
 ;;; Unfortunately this is a near total copy of the test in gc.impure.lisp
 (defun !ensure-genesis-code/data-separation ()
@@ -1299,29 +1402,51 @@ We could try a few things to mitigate this:
 ;;; this is a valid test that genesis separated code and data.
 (!ensure-genesis-code/data-separation)
 
-(defun hexdump (obj &optional (n-words
-                               (if (typep obj 'code-component)
-                                   ;; Display up through the first fun header
-                                   (+ (code-header-words obj)
-                                      (ash (sb-impl::%code-fun-offset obj 0)
-                                           (- word-shift))
-                                      simple-fun-code-offset)
-                                   ;; at most 10 words
-                                   (min 10 (ash (primitive-object-size obj)
-                                                (- word-shift)))))
-                              ;; pass NIL explicitly if T crashes on you
-                              (decode t))
-  (with-pinned-objects (obj)
-    (let ((a (logandc2 (get-lisp-obj-address obj) lowtag-mask)))
-      (dotimes (i n-words)
-        (let ((word (sap-ref-word (int-sap a) (ash i word-shift))))
+(defun hexdump (thing &optional (n-words nil wordsp)
+                                ;; pass NIL explicitly if T crashes on you
+                                (decode t))
+  (multiple-value-bind (obj addr count)
+      (if (integerp thing)
+          (values nil thing (if wordsp n-words 1))
+          (values
+           thing
+           (logandc2 (get-lisp-obj-address thing) lowtag-mask)
+           (if wordsp
+               n-words
+               (if (and (typep thing 'code-component) (plusp (code-n-entries thing)))
+                   ;; Display up through the first fun header
+                   (+ (code-header-words thing)
+                      (ash (%code-fun-offset thing 0) (- word-shift))
+                      simple-fun-insts-offset)
+                   ;; at most 16 words
+                   (min 16 (ash (primitive-object-size thing) (- word-shift)))))))
+    (with-pinned-objects (obj)
+      (dotimes (i count)
+        (let ((word (sap-ref-word (int-sap addr) (ash i word-shift))))
           (multiple-value-bind (lispobj ok) (if decode (make-lisp-obj word nil))
             (let ((*print-lines* 1)
                   (*print-pretty* t))
               (format t "~x: ~v,'0x~:[~; = ~S~]~%"
-                      (+ a (ash i word-shift))
+                      (+ addr (ash i word-shift))
                       (* 2 n-word-bytes)
                       word ok lispobj))))))))
+#+sb-thread
+(defun show-tls-map ()
+  (let ((list
+         (sort (sb-vm::list-allocated-objects
+                :all
+                :type sb-vm:symbol-widetag
+                :test (lambda (x) (plusp (sb-kernel:symbol-tls-index x))))
+               #'<
+               :key #'sb-kernel:symbol-tls-index))
+        (prev 0))
+    (dolist (x list)
+      (let ((n  (ash (sb-kernel:symbol-tls-index x) (- sb-vm:word-shift))))
+        (when (and (> n sb-thread::tls-index-start)
+                   (> n (1+ prev)))
+          (format t "(unused)~%"))
+        (format t "~5d = ~s~%" n x)
+        (setq prev n)))))
 
 (in-package "SB-C")
 ;;; As soon as practical in warm build it makes sense to add
