@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!THREAD")
+(in-package "SB-THREAD")
 
 (!define-thread-local *current-thread* nil
       "Bound in each thread to the thread itself.")
@@ -21,7 +21,7 @@
   "Type of native threads which are attached to the runtime as Lisp threads
 temporarily.")
 
-#!+(and sb-safepoint-strictly (not win32))
+#+(and sb-safepoint-strictly (not win32))
 (defstruct (signal-handling-thread
              (:copier nil)
              (:include foreign-thread)
@@ -38,11 +38,11 @@ stale value, use MUTEX-OWNER instead."
   "Test whether the current thread is holding MUTEX."
   ;; This is about the only use for which a stale value of owner is
   ;; sufficient.
-  (eq sb!thread:*current-thread* (mutex-%owner mutex)))
+  (eq sb-thread:*current-thread* (mutex-%owner mutex)))
 
 (defsetf mutex-value set-mutex-value)
 
-(declaim (sb!ext:deprecated :final ("SBCL" "1.2.15") #'set-mutex-value))
+(declaim (sb-ext:deprecated :final ("SBCL" "1.2.15") #'set-mutex-value))
 
 ;;; SPINLOCK no longer exists as a type -- provided for backwards compatibility.
 
@@ -50,7 +50,7 @@ stale value, use MUTEX-OWNER instead."
   "Spinlock type."
   'mutex)
 
-(declaim (sb!ext:deprecated
+(declaim (sb-ext:deprecated
           :late ("SBCL" "1.0.53.11") (type spinlock :replacement mutex)))
 
 (define-deprecated-function :early "1.0.53.11" make-spinlock make-mutex (&key name)
@@ -79,7 +79,7 @@ stale value, use MUTEX-OWNER instead."
   `(with-mutex (,lock)
      ,@body))
 
-(declaim (sb!ext:deprecated
+(declaim (sb-ext:deprecated
           :early ("SBCL" "1.0.53.11")
           (function with-recursive-spinlock :replacement with-recursive-lock)
           (function with-spinlock :replacement with-mutex)))
@@ -167,7 +167,7 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
                 (declare (function function))
                 (declare (dynamic-extent function))
                 (flet ((%call-with-system-mutex ()
-                         (dx-let (got-it)
+                         (let (got-it)
                            (unwind-protect
                                 (when (setf got-it (grab-mutex mutex))
                                   (funcall function))
@@ -186,7 +186,7 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
   (def call-with-system-mutex :without-gcing)
   (def call-with-system-mutex :allow-with-interrupts))
 
-#!-sb-thread
+#-sb-thread
 (progn
   (defun call-with-mutex (function mutex value waitp timeout)
     (declare (ignore mutex waitp timeout)
@@ -204,17 +204,9 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
   (defun call-with-recursive-system-lock (function lock)
     (declare (function function) (ignore lock))
     (without-interrupts
-      (funcall function)))
-
-  (defun call-with-recursive-system-lock/without-gcing (function mutex)
-    (declare (function function) (ignore mutex))
-    (without-gcing
       (funcall function))))
 
-#!+sb-thread
-;;; KLUDGE: These need to use DX-LET, because the cleanup form that
-;;; closes over GOT-IT causes a value-cell to be allocated for it --
-;;; and we prefer that to go on the stack since it can.
+#+sb-thread
 (progn
   (defun call-with-mutex (function mutex value waitp timeout)
     (declare (function function))
@@ -222,7 +214,7 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
     (unless (or (null value) (eq *current-thread* value))
       (error "~S called with non-nil :VALUE that isn't the current thread."
              'with-mutex))
-    (dx-let ((got-it nil))
+    (let ((got-it nil))
       (without-interrupts
         (unwind-protect
              (when (setq got-it (allow-with-interrupts
@@ -235,8 +227,8 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
   (defun call-with-recursive-lock (function mutex waitp timeout)
     (declare (function function))
     (declare (dynamic-extent function))
-    (dx-let ((inner-lock-p (eq (mutex-%owner mutex) *current-thread*))
-             (got-it nil))
+    (let ((inner-lock-p (eq (mutex-%owner mutex) *current-thread*))
+          (got-it nil))
       (without-interrupts
         (unwind-protect
              (when (or inner-lock-p (setf got-it (allow-with-interrupts
@@ -246,26 +238,15 @@ held mutex, WITH-RECURSIVE-LOCK allows recursive lock attempts to succeed."
           (when got-it
             (release-mutex mutex))))))
 
-  (macrolet ((def (name &optional variant)
-               `(defun ,(if variant (symbolicate name "/" variant) name)
-                    (function lock)
-                  (declare (function function))
-                  (declare (dynamic-extent function))
-                  (flet ((%call-with-recursive-system-lock ()
-                           (dx-let ((inner-lock-p
-                                     (eq *current-thread* (mutex-owner lock)))
-                                    (got-it nil))
-                             (unwind-protect
-                                  (when (or inner-lock-p
-                                            (setf got-it (grab-mutex lock)))
-                                    (funcall function))
-                               (when got-it
-                                 (release-mutex lock))))))
-                    (declare (inline %call-with-recursive-system-lock))
-                    ,(ecase variant
-                      (:without-gcing
-                        `(without-gcing (%call-with-recursive-system-lock)))
-                      ((nil)
-                        `(without-interrupts (%call-with-recursive-system-lock))))))))
-    (def call-with-recursive-system-lock)
-    (def call-with-recursive-system-lock :without-gcing)))
+  (defun call-with-recursive-system-lock (function lock)
+    (declare (function function))
+    (declare (dynamic-extent function))
+    (without-interrupts
+      (let ((inner-lock-p (eq *current-thread* (mutex-owner lock)))
+            (got-it nil))
+        (unwind-protect
+             (when (or inner-lock-p
+                       (setf got-it (grab-mutex lock)))
+               (funcall function))
+          (when got-it
+            (release-mutex lock)))))))
