@@ -43,20 +43,50 @@
         (cons (and (find-package "SB-INTERPRETER") value1)
               value2)))))
 
-(defvar *make-host-parallelism*
-  (or #+sbcl
-      (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
-        (when envvar
-          (require :sb-posix)
-          (parse-make-host-parallelism envvar)))))
-
+(defvar *make-host-parallelism* nil)
 (defun make-host-1-parallelism () (car *make-host-parallelism*))
 (defun make-host-2-parallelism () (cdr *make-host-parallelism*))
 
 #+sbcl
-(let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
-           (if (eq access :external) sym 'sb-unix:unix-exit))))
-  (defun exit-process (arg) (funcall f arg)))
+(progn
+  (setq *make-host-parallelism*
+        (let ((envvar (sb-ext:posix-getenv "SBCL_MAKE_PARALLEL")))
+          (when envvar
+            (require :sb-posix)
+            (parse-make-host-parallelism envvar))))
+  (defmacro with-subprocesses (&rest body) `(progn ,@body))
+  (let ((f (multiple-value-bind (sym access) (find-symbol "OS-EXIT" "SB-SYS")
+             (if (eq access :external) sym 'sb-unix:unix-exit))))
+    (defun exit-process (arg) (funcall f arg))
+    (defun exit-subprocess (arg) (funcall f arg)))
+  ;; Lazily reference sb-posix because it may not be loaded
+  (defun posix-fork () (funcall (intern "FORK" "HOST-SB-POSIX")))
+  (defun getpid () (funcall (intern "UNIX-GETPID" "HOST-SB-UNIX")))
+  (defun posix-wait () (funcall (intern "WAIT" "HOST-SB-POSIX"))))
+
+#+clisp
+(progn
+  (setq *make-host-parallelism*
+        (let ((envvar (ext:getenv "SBCL_MAKE_PARALLEL")))
+          (when envvar
+            (parse-make-host-parallelism envvar))))
+  ;; FFI symbols won't exist if libffcall could not be found at build time.
+  (defmacro with-subprocesses (&rest rest)
+    (cons (or (find-symbol "WITH-SUBPROCESSES" "POSIX") 'progn) rest))
+  ;; clisp doesn't expose fork() and consequently doesn't behave
+  ;; correctly when (EXT:EXIT) is called in a forked child process.
+  (defun exit-process (arg) (ext:exit arg))
+  #+#.(cl:if (cl:find-package "FFI") '(and) '(or))
+  (progn (ffi:def-call-out exit-subprocess (:name "exit") (:arguments (arg ffi:int))
+                  (:library :default) (:language :stdc))
+         (ffi:def-call-out posix-fork (:name "fork") (:return-type ffi:int)
+                  (:library :default) (:language :stdc)))
+  (defun getpid () (posix:process-id))
+  (defun posix-wait ()
+    (multiple-value-bind (pid status code) (posix:wait)
+      (if (eql status :exited)
+          (values pid code)
+          (values pid (- code))))))
 
 ;;; If TRUE, then COMPILE-FILE is being invoked only to process
 ;;; :COMPILE-TOPLEVEL forms, not to produce an output file.
@@ -251,7 +281,7 @@
 (let ((feature-compatibility-tests
        '(("(and sb-thread (not gencgc))"
           ":SB-THREAD requires :GENCGC")
-         ("(and sb-thread (not (or ppc x86 x86-64 arm64)))"
+         ("(and sb-thread (not (or ppc ppc64 x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
          ("(and gencgc cheneygc)"
           ":GENCGC and :CHENEYGC are incompatible")
@@ -389,13 +419,16 @@
 ;;; Given a STEM, remap the path component "/target/" to a suitable
 ;;; target directory.
 (defun stem-remap-target (stem)
-  (let ((position (search "/target/" stem)))
-    (if position
-      (concatenate 'string
-                   (subseq stem 0 (1+ position))
-                   (string-downcase (target-platform-keyword))
-                   (subseq stem (+ position 7)))
-      stem)))
+  (flet ((try-replacing (this that)
+           (let ((position (search this stem)))
+             (when position
+               (concatenate 'string
+                            (subseq stem 0 (1+ position))
+                            (string-downcase that)
+                            (subseq stem (+ position (length this) -1)))))))
+    (or (try-replacing "/target/" (target-platform-keyword))
+        (try-replacing "/asm-target/" (backend-assembler-target-name))
+        stem)))
 (compile 'stem-remap-target)
 
 ;;; Determine the source path for a stem by remapping from the abstract name
@@ -560,7 +593,8 @@
              (restart-case
                  (if trace-file
                      (funcall compile-file src :output-file tmp-obj
-                                               :trace-file t :allow-other-keys t)
+                              ;; if tracing, also show high-level progress
+                              :trace-file t :print t :allow-other-keys t)
                      (funcall compile-file src :output-file tmp-obj))
                (recompile ()
                  :report report-recompile-restart
@@ -688,8 +722,13 @@
              '(or))
   (labels ((contains-irrational (x)
              (typecase x
-               (cons (or (contains-irrational (car x))
-                         (contains-irrational (cdr x))))
+               (cons
+                ;; Tail-recursion not guaranteed
+                (do ((cons x (cdr cons)))
+                    ((atom cons)
+                     (contains-irrational cons))
+                  (when (contains-irrational (car cons))
+                    (return t))))
                (simple-vector (some #'contains-irrational x))
                ;; We use package literals -- see e.g. SANE-PACKAGE - which
                ;; must be treated as opaque, but COMMAs should not be opaque.
@@ -749,3 +788,32 @@
                     table stream))
          (format t "~&; wrote ~a - ~d entries"
                  filename (hash-table-count table))))))
+
+;;;; Please avoid writing "consecutive" (un-nested) reader conditionals
+;;;; in this file, whether for the same or different feature test.
+;;;; The following example prints 3 different results in 3 different lisp
+;;;; implementations all of which have feature :linux (and not :nofeat).
+#|
+(let ((i 0))
+  (dolist (s '("#-nofeat #+linux a b c d e"
+               "#-nofeat #-linux a b c d e"
+               "#+nofeat #-linux a b c d e"
+               "#+nofeat #+linux a b c d e"
+               "#+linux #-nofeat a b c d e"
+               "#-linux #-nofeat a b c d e"
+               "#-linux #+nofeat a b c d e"
+               "#+linux #+nofeat a b c d e"
+               "#+linux #-linux a b c d e"
+               "#-linux #-linux a b c d e"
+               "#-linux #+linux a b c d e"
+               "#+linux #+linux a b c d e"
+               "#+nofeat #-nofeat a b c d e"
+               "#-nofeat #-nofeat a b c d e"
+               "#-nofeat #+nofeat a b c d e"
+               "#+nofeat #+nofeat a b c d e"))
+  (format t "test ~2d: ~a ~s~%"
+          (incf i)
+          (let ((stream (make-string-input-stream s)))
+            (list (read stream) (read stream) (read stream)))
+          s)))
+|#
