@@ -13,27 +13,37 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
+
+;;; An OPAQUE-BOX instance is used to pass data from IR1 to IR2 as
+;;; a quoted object in a "source form" (not user-written) such that the
+;;; contained object is in a for-evaluation position but ignored by
+;;; the compiler's constant-dumping logic. In addition to this structure
+;;; type, a few other IR1 object types are implicitly opaque.
+(defstruct (opaque-box (:constructor opaquely-quote (value))
+                       (:copier nil)
+                       (:predicate opaque-box-p))
+  value)
 
 ;;; ANSI limits on compilation
-(defconstant sb!xc:call-arguments-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:call-arguments-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of arguments which may be passed
   to a function, including &REST args.")
-(defconstant sb!xc:lambda-parameters-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:lambda-parameters-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of parameters which may be specified
   in a given lambda list. This is actually the limit on required and &OPTIONAL
   parameters. With &KEY and &AUX you can get more.")
-(defconstant sb!xc:multiple-values-limit sb!xc:most-positive-fixnum
+(defconstant sb-xc:multiple-values-limit sb-xc:most-positive-fixnum
   "The exclusive upper bound on the number of multiple VALUES that you can
   return.")
 
 ;;;; cross-compiler-only versions of CL special variables, so that we
 ;;;; don't have weird interactions with the host compiler
 
-(defvar sb!xc:*compile-file-pathname*)
-(defvar sb!xc:*compile-file-truename*)
-(defvar sb!xc:*compile-print*)
-(defvar sb!xc:*compile-verbose*)
+(defvar sb-xc:*compile-file-pathname*)
+(defvar sb-xc:*compile-file-truename*)
+(defvar sb-xc:*compile-print*)
+(defvar sb-xc:*compile-verbose*)
 
 ;;;; miscellaneous types used both in the cross-compiler and on the target
 
@@ -41,33 +51,41 @@
 ;;;; somewhere else, not "early-c", since they're after all not part
 ;;;; of the compiler.
 
-;;; the type of LAYOUT-DEPTHOID slot values
-(def!type layout-depthoid () '(or index (integer -1 -1)))
+;;; the type of LAYOUT-DEPTHOID and LAYOUT-LENGTH values.
+;;; Each occupies two bytes of the %BITS slot when possible,
+;;; otherwise a slot unto itself.
+(def!type layout-depthoid () '(integer -1 #x7FFF))
+(def!type layout-length () '(integer 0 #xFFFF))
 (def!type layout-bitmap ()
   ;; FIXME: Probably should exclude negative bignum
-  #!+compact-instance-header 'integer
-  #!-compact-instance-header '(and integer (not (eql 0))))
+  #+compact-instance-header 'integer
+  #-compact-instance-header '(and integer (not (eql 0))))
 
 ;;; An INLINEP value describes how a function is called. The values
 ;;; have these meanings:
 ;;;     NIL     No declaration seen: do whatever you feel like, but don't
 ;;;             dump an inline expansion.
-;;; :NOTINLINE  NOTINLINE declaration seen: always do full function call.
-;;;    :INLINE  INLINE declaration seen: save expansion, expanding to it
+;;; NOTINLINE  NOTINLINE declaration seen: always do full function call.
+;;;    INLINE  INLINE declaration seen: save expansion, expanding to it
 ;;;             if policy favors.
-;;; :MAYBE-INLINE
+;;; MAYBE-INLINE
 ;;;             Retain expansion, but only use it opportunistically.
-;;;             :MAYBE-INLINE is quite different from :INLINE. As explained
+;;;             MAYBE-INLINE is quite different from INLINE. As explained
 ;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
 ;;;             instantiated once per component, INLINE - for all
 ;;;             references (even under #'without FUNCALL)."
-(deftype inlinep ()
-  '(member :inline :maybe-inline :notinline nil))
-(defconstant-eqx +inlinep-translations+
-  '((inline . :inline)
-    (notinline . :notinline)
-    (maybe-inline . :maybe-inline))
-  #'equal)
+(def!type inlinep ()
+  '(member inline maybe-inline notinline nil))
+
+(defstruct (dxable-args (:constructor make-dxable-args (list))
+                        (:predicate nil)
+                        (:copier nil))
+  (list nil :read-only t))
+(defstruct (inlining-data (:include dxable-args)
+                          (:constructor make-inlining-data (expansion list))
+                          (:predicate nil)
+                          (:copier nil))
+  (expansion nil :read-only t))
 
 ;;; *FREE-VARS* translates from the names of variables referenced
 ;;; globally to the LEAF structures for them. *FREE-FUNS* is like
@@ -89,43 +107,25 @@
 (defvar *allow-instrumenting*)
 
 ;;; miscellaneous forward declarations
-(defvar *code-segment*)
-;; FIXME: this is a kludge due to the absence of a 'vop' argument
-;; to ALLOCATION-TRAMP in the x86-64 backend.
-(defvar *code-is-immobile*)
-#!+sb-dyncount (defvar *collect-dynamic-statistics*)
+#+sb-dyncount (defvar *collect-dynamic-statistics*)
 (defvar *component-being-compiled*)
 (defvar *compiler-error-context*)
 (defvar *compiler-error-count*)
 (defvar *compiler-warning-count*)
 (defvar *compiler-style-warning-count*)
 (defvar *compiler-note-count*)
-(defvar *compiler-trace-output*)
+;;; Bind this to a stream to capture various internal debugging output.
+(defvar *compiler-trace-output* nil)
+(defvar *compile-trace-targets* '(:ir1 :ir2 :vop :disassemble))
 (defvar *constraint-universe*)
 (defvar *current-path*)
 (defvar *current-component*)
 (defvar *delayed-ir1-transforms*)
-(defvar *eval-tlf-index*)
+#+sb-dyncount
 (defvar *dynamic-counts-tn*)
-(defvar *elsewhere*)
 (defvar *elsewhere-label*)
-(defvar *event-info*)
 (defvar *event-note-threshold*)
 (defvar *failure-p*)
-(defvar *fixup-notes*)
-#!+inline-constants
-(progn
-  (defvar *unboxed-constants*)
-  (defstruct (unboxed-constants (:conc-name constant-)
-                                (:predicate nil) (:copier nil))
-    (table (make-hash-table :test #'equal) :read-only t)
-    (segment
-     (sb!assem:make-segment :type :elsewhere
-                            :run-scheduler nil
-                            :inst-hook (default-segment-inst-hook)
-                            :alignment 0) :read-only t)
-    (vector (make-array 16 :adjustable t :fill-pointer 0) :read-only t))
-  (declaim (freeze-type unboxed-constants)))
 (defvar *source-info*)
 (defvar *source-plist*)
 (defvar *source-namestring*)
@@ -133,6 +133,7 @@
 (defvar *warnings-p*)
 (defvar *lambda-conversions*)
 (defvar *compile-object* nil)
+(defvar *location-context* nil)
 
 (defvar *stack-allocate-dynamic-extent* t
   "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
@@ -141,26 +142,26 @@ possible. Potentially long (over one page in size) vectors are, however, not
 stack allocated except in zero SAFETY code, as such a vector could overflow
 the stack without triggering overflow protection.")
 
-(!begin-collecting-cold-init-forms)
 ;;; This lock is seized in the compiler, and related areas -- like the
 ;;; classoid/layout/class system.
-(defglobal **world-lock** nil)
-(!cold-init-forms
- (setf **world-lock** (sb!thread:make-mutex :name "World Lock")))
-(!defun-from-collected-cold-init-forms !world-lock-cold-init)
+;;; Assigning a literal object enables genesis to dump and load it
+;;; without need of a cold-init function.
+#-sb-xc-host
+(!define-load-time-global **world-lock** #.(sb-thread:make-mutex :name "World Lock"))
+
+#-sb-xc-host
+(define-load-time-global *static-linker-lock*
+    (sb-thread:make-mutex :name "static linker"))
 
 (defmacro with-world-lock (() &body body)
-  `(sb!thread:with-recursive-lock (**world-lock**)
-     ,@body))
-
-(declaim (type fixnum *compiler-sset-counter*))
-(defvar *compiler-sset-counter* 0)
+  #+sb-xc-host `(progn ,@body)
+  #-sb-xc-host `(sb-thread:with-recursive-lock (**world-lock**) ,@body))
 
 ;;; unique ID for the next object created (to let us track object
 ;;; identity even across GC, useful for understanding weird compiler
 ;;; bugs where something is supposed to be unique but is instead
 ;;; exists as duplicate objects)
-#!+sb-show
+#+sb-show
 (progn
   (defvar *object-id-counter* 0)
   (defun new-object-id ()
@@ -174,10 +175,11 @@ the stack without triggering overflow protection.")
 ;;; Rather than registering listeners, they can detect changes by comparing
 ;;; their stored nonce to the current nonce. Additionally the observers
 ;;; can detect whether function definitions have occurred.
-(declaim (fixnum *type-cache-nonce*))
-(!defglobal *type-cache-nonce* 0)
+#-sb-xc-host
+(progn (declaim (fixnum *type-cache-nonce*))
+       (!define-load-time-global *type-cache-nonce* 0))
 
-(def!struct (undefined-warning
+(defstruct (undefined-warning
             #-no-ansi-print-object
             (:print-object (lambda (x s)
                              (print-unreadable-object (x s :type t)
@@ -235,28 +237,16 @@ the stack without triggering overflow protection.")
                 :format-arguments (list symbol)))
   (values))
 
-(def!struct (debug-name-marker (:print-function print-debug-name-marker)
-                               (:copier nil)))
+(defstruct (debug-name-marker (:print-function print-debug-name-marker)
+                              ;; make these satisfy SB-XC:INSTANCEP
+                              #+sb-xc-host (:include structure!object)
+                              (:copier nil)))
 
 (defvar *debug-name-level* 4)
 (defvar *debug-name-length* 12)
 (defvar *debug-name-punt*)
-(defvar *debug-name-sharp*)
-(defvar *debug-name-ellipsis*)
-
-(defmethod make-load-form ((marker debug-name-marker) &optional env)
-  (declare (ignore env))
-  (cond ((eq marker *debug-name-sharp*)
-         `(if (boundp '*debug-name-sharp*)
-              *debug-name-sharp*
-              (make-debug-name-marker)))
-        ((eq marker *debug-name-ellipsis*)
-         `(if (boundp '*debug-name-ellipsis*)
-              *debug-name-ellipsis*
-              (make-debug-name-marker)))
-        (t
-         (warn "Dumping unknown debug-name marker.")
-         '(make-debug-name-marker))))
+(define-load-time-global *debug-name-sharp* (make-debug-name-marker))
+(define-load-time-global *debug-name-ellipsis* (make-debug-name-marker))
 
 (defun print-debug-name-marker (marker stream level)
   (declare (ignore level))
@@ -267,9 +257,6 @@ the stack without triggering overflow protection.")
         (t
          (write-string "???" stream))))
 
-(setf *debug-name-sharp* (make-debug-name-marker)
-      *debug-name-ellipsis* (make-debug-name-marker))
-
 (declaim (ftype (sfunction () list) name-context))
 (defun debug-name (type thing &optional context)
   (let ((*debug-name-punt* nil))
@@ -304,11 +291,94 @@ the stack without triggering overflow protection.")
                 debug name." name))
         name))))
 
-;;; Set this to NIL to inhibit assembly-level optimization. (For
-;;; compiler debugging, rather than policy control.)
-(defvar *assembly-optimize* t)
+;;; Bound during eval-when :compile-time evaluation.
+(defvar *compile-time-eval* nil)
+(declaim (always-bound *compile-time-eval*))
 
-(in-package "SB!ALIEN")
+#-immobile-code (defmacro code-immobile-p (thing) `(progn ,thing nil))
+
+;;; Various error-code generating helpers
+(defvar *adjustable-vectors*)
+
+(defmacro with-adjustable-vector ((var) &rest body)
+  `(let ((,var (or (pop *adjustable-vectors*)
+                   (make-array 16
+                               :element-type '(unsigned-byte 8)
+                               :fill-pointer 0
+                               :adjustable t))))
+     ;; Don't declare the length - if it gets adjusted and pushed back
+     ;; onto the freelist, it's anyone's guess whether it was expanded.
+     ;; This code was wrong for >12 years, so nobody must have needed
+     ;; more than 16 elements. Maybe we should make it nonadjustable?
+     (declare (type (vector (unsigned-byte 8)) ,var))
+     (setf (fill-pointer ,var) 0)
+     ;; No UNWIND-PROTECT here - semantics are unaffected by nonlocal exit,
+     ;; and this macro is about speeding up the compiler, not slowing it down.
+     ;; GC will clean up any debris, and since the vector does not point
+     ;; to anything, even an accidental promotion to a higher generation
+     ;; will not cause transitive garbage retention.
+     (prog1 (progn ,@body)
+       (push ,var *adjustable-vectors*))))
+
+
+;;; The allocation quantum for boxed code header words.
+;;; 2 implies an even length boxed header; 1 implies no restriction.
+(defconstant code-boxed-words-align (+ 2 #+(or x86 x86-64) -1))
+
+;;; Used as the CDR of the code coverage instrumentation records
+;;; (instead of NIL) to ensure that any well-behaving user code will
+;;; not have constants EQUAL to that record. This avoids problems with
+;;; the records getting coalesced with non-record conses, which then
+;;; get mutated when the instrumentation runs. Note that it's
+;;; important for multiple records for the same location to be
+;;; coalesced. -- JES, 2008-01-02
+(defconstant +code-coverage-unmarked+ '%code-coverage-unmarked%)
+
+;;; Stores the code coverage instrumentation results.
+;;; The CAR is a hashtable. The CDR is a list of weak pointers to code objects
+;;; having coverage marks embedded in the unboxed constants.
+;;; Keys in the hashtable are namestrings, the
+;;; value is a list of (CONS PATH STATE), where STATE is +CODE-COVERAGE-UNMARKED+
+;;; for a path that has not been visited, and T for one that has.
+#-sb-xc-host
+(progn
+  (define-load-time-global *code-coverage-info*
+    (list (make-hash-table :test 'equal :synchronized t)))
+  (declaim (type (cons hash-table) *code-coverage-info*)))
+
+(defstruct (compilation (:copier nil)
+                        (:predicate nil)
+                        (:conc-name ""))
+  (fun-names-in-this-file)
+  (coverage-metadata nil :type (or (cons hash-table hash-table) null) :read-only t)
+  (msan-unpoison nil :read-only t)
+  (sset-counter 1 :type fixnum)
+  ;; Bidrectional map between IR1/IR2/assembler abstractions
+  ;; and a corresponding small integer identifier. One direction could be done
+  ;; by adding the integer ID as an object slot, but we want both directions.
+  ;; These could just as well be scoped by WITH-IR1-NAMESPACE, but
+  ;; since it's primarily a debugging tool, it's nicer to have
+  ;; a wider unique scope by ID.
+  (objmap-obj-to-id   (make-hash-table :test 'eq) :read-only t)
+  (objmap-id-to-cont  (make-array 10) :type simple-vector) ; number -> CTRAN or LVAR
+  (objmap-id-to-tn    (make-array 10) :type simple-vector) ; number -> TN
+  (objmap-id-to-label (make-array 10) :type simple-vector) ; number -> LABEL
+  (objmap-cont-num    0 :type fixnum)
+  (objmap-tn-id       0 :type fixnum)
+  (objmap-label-id    0 :type fixnum)
+  ;; if emitting a cfasl, the fasl stream to that
+  (compile-toplevel-object nil :read-only t)
+  ;; these are all historical baggage from here down,
+  ;; unused unless we ever decide to fix block compilation
+  (block-compile nil :type (member nil t :specified))
+  ;; When block compiling, used by PROCESS-FORM to accumulate top level
+  ;; lambdas resulting from compiling subforms. (In reverse order.)
+  (toplevel-lambdas nil :type list))
+
+(defvar *compilation*)
+(declaim (type compilation *compilation*))
+
+(in-package "SB-ALIEN")
 
 ;;; Information describing a heap-allocated alien.
 (def!struct (heap-alien-info (:copier nil))
@@ -319,3 +389,8 @@ the stack without triggering overflow protection.")
   ;; Data or code?
   (datap (missing-arg) :type boolean))
 (!set-load-form-method heap-alien-info (:xc :target))
+
+;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
+;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
+#+linux ; shadow space differs by OS
+(defconstant sb-vm::msan-mem-to-shadow-xor-const #x500000000000)
