@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; source-hacking defining forms
 
@@ -35,7 +35,7 @@
                                  :wrap-block nil)))
     (declare (ignorable doc)) ; unused on host
     ;; Maybe kill docstring, but only under the cross-compiler.
-    #!+(and (not sb-doc) (host-feature sb-xc-host)) (setq doc nil)
+    #+(and (not sb-doc) sb-xc-host) (setq doc nil)
     `(progn
       (declaim (ftype (function (ctran ctran (or lvar null) t) (values))
                       ,fn-name))
@@ -286,8 +286,8 @@
     (error "can't specify both DEFUN-ONLY and EVAL-NAME"))
   (multiple-value-bind (body decls doc) (parse-body body-decls-doc t)
     (let ((n-node (or node (make-symbol "NODE")))
-          (n-decls (sb!xc:gensym))
-          (n-lambda (sb!xc:gensym)))
+          (n-decls (sb-xc:gensym))
+          (n-lambda (sb-xc:gensym)))
       (multiple-value-bind (bindings vars)
           (parse-deftransform lambda-list n-node
                               '(give-up-ir1-transform))
@@ -304,14 +304,15 @@
                   ;; What purpose does it serve to allow the transform's body
                   ;; to return decls as a second value? They would go in the
                   ;; right place if simply returned as part of the expression.
-                  (multiple-value-bind (,n-lambda ,n-decls)
+                  (block ,(fun-name-block-name name)
+                   (multiple-value-bind (,n-lambda ,n-decls)
                       (progn ,@body)
                     (if (and (consp ,n-lambda) (eq (car ,n-lambda) 'lambda))
                         ,n-lambda
                         `(lambda ,',lambda-list
                            (declare (ignorable ,@',vars))
                            ,@,n-decls
-                           ,,n-lambda))))))
+                           ,,n-lambda)))))))
           (if defun-only
               `(defun ,name ,@stuff)
               `(%deftransform
@@ -374,11 +375,10 @@
 ;;; methods are passed an additional POLICY argument, and IR2-CONVERT
 ;;; methods are passed an additional IR2-BLOCK argument.
 (defmacro defoptimizer (what (lambda-list
-                              &optional (node (sb!xc:gensym) node-p)
+                              &optional (node (sb-xc:gensym) node-p)
                               &rest vars)
                         &body body)
-  (binding* ((name
-              (flet ((function-name (name)
+  (let ((name (flet ((function-name (name)
                        (etypecase name
                          (symbol name)
                          ((cons (eql setf) (cons symbol null))
@@ -386,32 +386,44 @@
                 (if (symbolp what)
                     what
                     (symbolicate (function-name (first what))
-                                 "-" (second what) "-OPTIMIZER"))))
-             ((forms decls) (parse-body body nil))
-             ((var-decls more-decls) (extract-var-decls decls vars))
-             ;; In case the BODY declares IGNORE of the formal NODE var,
-             ;; we rebind it from N-NODE and never reference it from BINDS.
-             (n-node (make-symbol "NODE"))
-             ((binds lambda-vars gensyms)
-              (parse-deftransform lambda-list n-node
-                                  `(return-from ,name nil))))
-    (declare (ignore lambda-vars))
-    `(progn
-       ;; We can't stuff the BINDS as &AUX vars into the lambda list
-       ;; because there can be a RETURN-FROM in there.
-       (defun ,name (,n-node ,@vars)
-         ,@(if var-decls (list var-decls))
-         (let* (,@binds ,@(if node-p `((,node ,n-node))))
-           ;; Syntax requires naming NODE even if undesired if VARS
-           ;; are present, so in that case make NODE ignorable.
-           (declare (ignorable ,@(if (and vars node-p) `(,node))
-                               ,@gensyms))
-           ,@more-decls ,@forms))
-       ,@(when (consp what)
-           `((setf (,(let ((*package* (symbol-package 'fun-info)))
-                          (symbolicate "FUN-INFO-" (second what)))
-                       (fun-info-or-lose ',(first what)))
-                      #',name))))))
+                                 "-"
+                                 (if (consp (second what))
+                                     (caadr what)
+                                     (second what))
+                                 "-OPTIMIZER")))))
+    (if (typep what '(cons (eql vop-optimize)))
+        `(progn
+           (defun ,name (,lambda-list)
+             ,@body)
+           ,@(loop for what in (ensure-list (second what))
+                   collect
+                   `(setf (vop-info-optimizer (template-or-lose ',what))
+                          #',name)))
+        (binding* (((forms decls) (parse-body body nil))
+                   ((var-decls more-decls) (extract-var-decls decls vars))
+                   ;; In case the BODY declares IGNORE of the formal NODE var,
+                   ;; we rebind it from N-NODE and never reference it from BINDS.
+                   (n-node (make-symbol "NODE"))
+                   ((binds lambda-vars gensyms)
+                    (parse-deftransform lambda-list n-node
+                                        `(return-from ,name nil))))
+          (declare (ignore lambda-vars))
+          `(progn
+             ;; We can't stuff the BINDS as &AUX vars into the lambda list
+             ;; because there can be a RETURN-FROM in there.
+             (defun ,name (,n-node ,@vars)
+               ,@(if var-decls (list var-decls))
+               (let* (,@binds ,@(if node-p `((,node ,n-node))))
+                 ;; Syntax requires naming NODE even if undesired if VARS
+                 ;; are present, so in that case make NODE ignorable.
+                 (declare (ignorable ,@(if (and vars node-p) `(,node))
+                                     ,@gensyms))
+                 ,@more-decls ,@forms))
+             ,@(when (consp what)
+                 `((setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
+                             (symbolicate "FUN-INFO-" (second what)))
+                          (fun-info-or-lose ',(first what)))
+                         #',name))))))))
 
 ;;;; IR groveling macros
 
@@ -605,11 +617,11 @@
 ;;; SLOT, returning the <value, T>, or <NIL, NIL> if no entry. The
 ;;; :TEST keyword may be used to determine the name equality
 ;;; predicate.
-(defmacro lexenv-find (name slot &key test)
-  (once-only ((n-res `(assoc ,name (,(let ((*package* (symbol-package 'lexenv-funs)))
-                                          (symbolicate "LEXENV-" slot))
-                                     *lexenv*)
-                             :test ,(or test '#'eq))))
+(defmacro lexenv-find (name slot &key (lexenv '*lexenv*) test
+                                 &aux (accessor (package-symbolicate
+                                                 (cl:symbol-package 'lexenv-funs)
+                                                 "LEXENV-" slot)))
+  (once-only ((n-res `(assoc ,name (,accessor ,lexenv) :test ,(or test '#'eq))))
     `(if ,n-res
          (values (cdr ,n-res) t)
          (values nil nil))))
@@ -698,13 +710,14 @@
 ;;; event happens.
 (defmacro defevent (name description &optional (level 0))
   (let ((var-name (symbolicate "*" name "-EVENT-INFO*")))
-    `(eval-when (:compile-toplevel :load-toplevel :execute)
-       (define-load-time-global ,var-name
+    `(progn
+       (defglobal ,var-name
          (make-event-info :name ',name
                           :description ',description
                           :var ',var-name
                           :level ,level))
-       (setf (gethash ',name *event-info*) ,var-name)
+       (eval-when (:compile-toplevel :load-toplevel :execute)
+         (setf (gethash ',name *event-info*) ,var-name))
        ',name)))
 
 ;;; the lowest level of event that will print a note when it occurs
@@ -746,7 +759,7 @@
 ;;;; functions on directly-linked lists (linked through specialized
 ;;;; NEXT operations)
 
-#!-sb-fluid (declaim (inline find-in position-in))
+#-sb-fluid (declaim (inline find-in position-in))
 
 ;;; Find ELEMENT in a null-terminated LIST linked by the accessor
 ;;; function NEXT. KEY and TEST are the same as for generic sequence functions.
@@ -782,7 +795,7 @@
 
 (defmacro deletef-in (next place item &environment env)
   (multiple-value-bind (temps vals stores store access)
-      (#+sb-xc sb!xc:get-setf-expansion #-sb-xc get-setf-expansion place env)
+      (#+sb-xc sb-xc:get-setf-expansion #-sb-xc get-setf-expansion place env)
     (when (cdr stores)
       (error "multiple store variables for ~S" place))
     (let ((n-item (gensym))
@@ -808,7 +821,7 @@
 ;;;
 (defmacro push-in (next item place &environment env)
   (multiple-value-bind (temps vals stores store access)
-      (#+sb-xc sb!xc:get-setf-expansion #-sb-xc get-setf-expansion place env)
+      (#+sb-xc sb-xc:get-setf-expansion #-sb-xc get-setf-expansion place env)
     (when (cdr stores)
       (error "multiple store variables for ~S" place))
     `(let (,@(mapcar #'list temps vals)
