@@ -18,10 +18,6 @@
 ;;; Set to NIL to disable loop analysis for register allocation.
 (defvar *loop-analyze* t)
 
-;;; Bind this to a stream to capture various internal debugging output.
-(defvar *compiler-trace-output* nil)
-(defvar *compile-trace-targets* '(:ir1 :ir2 :vop :disassemble))
-
 ;;; *BLOCK-COMPILE-ARG* holds the original value of the :BLOCK-COMPILE
 ;;; argument, which overrides any internal declarations.
 (defvar *block-compile-arg*)
@@ -234,21 +230,20 @@ Examples:
 (defun summarize-compilation-unit (abort-p)
   (let (summary)
     (unless abort-p
-      (handler-bind ((style-warning #'compiler-style-warning-handler)
-                     (warning #'compiler-warning-handler))
-
-        (let ((undefs (sort *undefined-warnings* #'string<
-                            :key (lambda (x)
-                                   (let ((x (undefined-warning-name x)))
-                                     (if (symbolp x)
-                                         (symbol-name x)
-                                         (prin1-to-string x))))))
-              (*last-message-count* (list* 0 nil nil))
-              (*last-error-context* nil))
+      (let ((undefs (sort *undefined-warnings* #'string<
+                          :key (lambda (x)
+                                 (let ((x (undefined-warning-name x)))
+                                   (if (symbolp x)
+                                       (symbol-name x)
+                                       (prin1-to-string x))))))
+            (*last-message-count* (list* 0 nil nil))
+            (*last-error-context* nil))
+        (handler-bind ((style-warning #'compiler-style-warning-handler)
+                       (warning #'compiler-warning-handler))
           (dolist (kind '(:variable :function :type))
             (let ((names (mapcar #'undefined-warning-name
-                                   (remove kind undefs :test #'neq
-                                           :key #'undefined-warning-kind))))
+                                 (remove kind undefs :test #'neq
+                                                     :key #'undefined-warning-kind))))
               (when names (push (cons kind names) summary))))
           (dolist (undef undefs)
             (let ((name (undefined-warning-name undef))
@@ -562,6 +557,7 @@ necessary, since type inference may take arbitrarily long to converge.")
       ;; STACK only uses dominance information for DX LVAR back
       ;; propagation (see BACK-PROPAGATE-ONE-DX-LVAR).
       (when (component-dx-lvars component)
+        (clear-dominators component)
         (find-dominators component))
       (stack-analyze component)
       ;; Assign BLOCK-NUMBER for any cleanup blocks introduced by
@@ -633,7 +629,7 @@ necessary, since type inference may take arbitrarily long to converge.")
                 (let ((ranges
                         (maplist (lambda (list)
                                    (cons (+ (car list)
-                                            (ash sb-vm:simple-fun-code-offset
+                                            (ash sb-vm:simple-fun-insts-offset
                                                  sb-vm:word-shift))
                                          (or (cadr list) text-length)))
                                  fun-table)))
@@ -1091,7 +1087,8 @@ necessary, since type inference may take arbitrarily long to converge.")
   (let ((*top-level-form-noted* (note-top-level-form form t)))
     ;; Don't bother to compile simple objects that just sit there.
     (when (and form (or (symbolp form) (consp form)))
-      (if (and (policy *policy*
+      (if (and #-sb-xc-host
+               (policy *policy*
                    ;; FOP-compiled code is harder do debug.
                    (or (< debug 2)
                        (> space debug)))
@@ -1438,7 +1435,8 @@ necessary, since type inference may take arbitrarily long to converge.")
                   (when compile-time-too
                     (eval-compile-toplevel (list form) path))
                   (cond ((deferrable-tlf-p form)
-                         (push (vector *source-paths* *policy* *handled-conditions*
+                         (push (vector (copy-hash-table *source-paths*)
+                                       *policy* *handled-conditions*
                                        *disabled-package-locks* *lexenv* form path)
                                (queued-tlfs)))
                         (t
@@ -1449,6 +1447,14 @@ necessary, since type inference may take arbitrarily long to converge.")
                            (convert-and-maybe-compile form path)))))))))))
 
   (values))
+
+(defun copy-hash-table (hash-table)
+  (let ((new (make-hash-table :test (hash-table-test hash-table)
+                              :size (hash-table-size hash-table))))
+    (maphash (lambda (key value)
+               (setf (gethash key new) value))
+             hash-table)
+    new))
 
 ;;;; load time value support
 ;;;;
