@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; Return the template having the specified name, or die trying.
 (defun template-or-lose (x)
@@ -21,11 +21,11 @@
 ;;; Return the SC structure, SB structure or SC number corresponding
 ;;; to a name, or die trying.
 (defun sc-or-lose (x)
-  (the sc
+  (the storage-class
        (or (gethash x *backend-sc-names*)
            (error "~S is not a defined storage class." x))))
 (defun sb-or-lose (x)
-  (the sb
+  (the storage-base
        (dovector (sb *backend-sbs*
                      (error "~S is not a defined storage base." x))
          (when (eq (sb-name sb) x)
@@ -44,7 +44,7 @@
 ;;; Compute at compiler load time the costs for moving between all SCs that
 ;;; can be loaded from FROM-SC and to TO-SC given a base move cost Cost.
 (defun compute-move-costs (from-sc to-sc cost)
-  (declare (type sc from-sc to-sc) (type index cost))
+  (declare (type storage-class from-sc to-sc) (type index cost))
   (let ((to-scn (sc-number to-sc))
         (from-costs (sc-load-costs from-sc)))
     (dolist (dest-sc (cons to-sc (sc-alternate-scs to-sc)))
@@ -73,7 +73,7 @@
 ;;; Return true if SC is either one of PTYPE's SC's, or one of those
 ;;; SC's alternate or constant SCs.
 (defun sc-allowed-by-primitive-type (sc ptype)
-  (declare (type sc sc) (type primitive-type ptype))
+  (declare (type storage-class sc) (type primitive-type ptype))
   (let ((scn (sc-number sc)))
     (dolist (allowed (primitive-type-scs ptype) nil)
       (when (eql allowed scn)
@@ -93,8 +93,6 @@
   ;; See also the description of VOP-INFO-TARGETS. -- APD, 2002-01-30
   (defconstant max-vop-tn-refs 256))
 
-(defconstant sc-bits (integer-length (1- sc-number-limit)))
-
 ;;; Emit a VOP for TEMPLATE. Arguments:
 ;;; NODE Node for source context.
 ;;; BLOCK IR2-BLOCK that we place the VOP in.
@@ -141,9 +139,10 @@
         (dotimes (i (length temps))
           (let* ((temp (aref temps i))
                  (tn (if (logbitp 0 temp)
-                         (make-wired-tn nil
-                                        (ldb (byte sc-bits 1) temp)
-                                        (ash temp (- (1+ sc-bits))))
+                         (make-wired-tn
+                          nil
+                          (ldb (byte sb-vm:sc-number-bits 1) temp)
+                          (ash temp (- (1+ sb-vm:sc-number-bits))))
                          (make-restricted-tn nil (ash temp -1))))
                  (write-ref (reference-tn tn t)))
                      ;; KLUDGE: These formulas must be consistent with
@@ -178,7 +177,7 @@
       (when targets
         (dotimes (i (length targets))
           (let ((target (aref targets i)))
-            (sb!regalloc:target-if-desirable
+            (sb-regalloc:target-if-desirable
                     (aref refs (ldb (byte 8 8) target))
                     (aref refs (ldb (byte 8 0) target)))))))
     vop))
@@ -197,7 +196,7 @@
 
 ;;; Return a function type specifier describing TEMPLATE's type computed
 ;;; from the operand type restrictions.
-#!-sb-fluid (declaim (inline template-conditional-p))
+#-sb-fluid (declaim (inline template-conditional-p))
 (defun template-conditional-p (template)
   (declare (type template template))
   (let ((rtypes (template-result-types template)))
