@@ -168,46 +168,9 @@ lose:
     return core_start;
 }
 
-#ifndef LISP_FEATURE_HPUX
-#define load_core_bytes(fd, where, addr, len) os_map(fd, where, addr, len)
-#else
-#define load_core_bytes(fd, where, addr, len) copy_core_bytes(fd, where, addr, len)
-/* If more platforms don't support overlapping mmap rename this
- * def to something like ifdef nommapoverlap */
-/* currently hpux only */
-static void copy_core_bytes(int fd, os_vm_offset_t offset,
-                            os_vm_address_t addr, int len)
-{
-  unsigned char buf[4096];
-  int c,x;
-  int old_fd = lseek(fd, 0, SEEK_CUR);
-
-  if(len & (4096-1)){
-    fprintf(stderr, "cant copy a slice of core because slice-length is not of page size(4096)\n");
-    exit(-1);
-  }
-  if(old_fd < 0){
-    fprintf(stderr, "cant perform lseek() on corefile\n");
-  }
-  lseek(fd, offset, SEEK_SET);
-  if(fd < 0){
-    fprintf(stderr, "cant perform lseek(%u,%lu,SEEK_SET) on corefile\n", fd, offset);
-  }
-  for(x = 0; x < len; x += 4096){
-    c = read(fd, buf, 4096);
-    if(c != 4096){
-      fprintf(stderr, "cant read memory area from corefile at position %lu, got %d\n", offset + x, c);
-      exit(-1);
-    }
-    memcpy(addr+x, buf, 4096);
-  }
-  os_flush_icache(addr, len);
-}
-#endif
-
 #ifndef LISP_FEATURE_SB_CORE_COMPRESSION
 # define inflate_core_bytes(fd,offset,addr,len) \
-    lose("This runtime was not built with zlib-compressed core support... aborting\n")
+    lose("This runtime was not built with zlib-compressed core support... aborting")
 #else
 # define ZLIB_BUFFER_SIZE (1u<<16)
 static void inflate_core_bytes(int fd, os_vm_offset_t offset,
@@ -224,7 +187,7 @@ static void inflate_core_bytes(int fd, os_vm_offset_t offset,
 # endif
 
     if (-1 == lseek(fd, offset, SEEK_SET)) {
-        lose("Unable to lseek() on corefile\n");
+        lose("Unable to lseek() on corefile");
     }
 
     stream.zalloc = NULL;
@@ -235,14 +198,14 @@ static void inflate_core_bytes(int fd, os_vm_offset_t offset,
 
     ret = inflateInit(&stream);
     if (ret != Z_OK)
-        lose("zlib error %i\n", ret);
+        lose("zlib error %i", ret);
 
     stream.next_out  = (void*)addr;
     stream.avail_out = len;
     do {
         ssize_t count = read(fd, buf, ZLIB_BUFFER_SIZE);
         if (count < 0)
-            lose("unable to read core file (errno = %i)\n", errno);
+            lose("unable to read core file (errno = %i)", errno);
         stream.next_in = buf;
         stream.avail_in = count;
         if (count == 0) break;
@@ -252,13 +215,13 @@ static void inflate_core_bytes(int fd, os_vm_offset_t offset,
             break;
         case Z_OK:
             if (stream.avail_out == 0)
-                lose("Runaway gzipped core directory... aborting\n");
+                lose("Runaway gzipped core directory... aborting");
             if (stream.avail_in > 0)
                 lose("zlib inflate returned without fully"
-                     "using up input buffer... aborting\n");
+                     "using up input buffer... aborting");
             break;
         default:
-            lose("zlib inflate error: %i\n", ret);
+            lose("zlib inflate error: %i", ret);
             break;
         }
     } while (ret != Z_STREAM_END);
@@ -289,14 +252,6 @@ struct heap_adjust {
     int n_relocs_rel; // relative
 };
 
-static inline struct code* get_asm_routine_code_component() {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-    return (struct code*)VARYOBJ_SPACE_START;
-#else
-    return (struct code*)READ_ONLY_SPACE_START;
-#endif
-}
-
 #ifndef LISP_FEATURE_RELOCATABLE_HEAP
 #define adjust_word(ignore,thing) thing
 #define relocate_heap(ignore)
@@ -437,37 +392,6 @@ static inline void fix_fun_header_layout(lispobj __attribute__((unused)) *fun,
 #endif
 }
 
-/* About relocation of fdefns - suppose we had originally intended to load:
- *   [ range of adjustable addresses  ]
- *   | fixedobj space | varyobj space |
- *          fdefn --------> callee
- *   0x1000000        0x2000000
- * and the subspaces were actually placed at much higher addresses
- * that do not overlap at all with the intended addresses:
- *                                       | fixedoboj space | varyobj space |
- *                                       0x8000000 ...
- * Then given an fdefn, its callee as computed from the new address appears
- * such there would be no relocation adjustment for the addressed object.
- * We correctly do not adjust the fdefn's callee displacement.
- * But suppose the moved spaces overlap the expected load addresses:
- *        [ range of adjustable addresses  ]
- *        | fixedobj space | varyobj space |
- *               fdefn --------> callee
- *   0x1000000 = desired
- *        0x1008000 = actual
- * In that case there can be an fdefn whose callee appears to be in the
- * adjustment range, and we'd compute an (incorrect) adjustment. In reality,
- * both the fdefn and its intended target moved by the same amount,
- * so the net change to the fdefn JMP instruction displacement ought to be zero.
- * To deal with this, we have to compute the fdefn callee as if the fdefn were
- * at its intended load address, then check whether that callee needs adjustment,
- * and compute a net adjustment by incorporating the amount by which the fdefn
- * itself moved.  Since both spaces move together for now, these two deltas
- * cancel out, and the net adjustment becomes zero. The full complexity is
- * required to handle the situation where only one of the immobile subspaces
- * moved, which we can't actually handle yet at all.
- * But already there was a bug when they move together by a small amount.
- */
 static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
     lispobj *where = (lispobj*)start;
@@ -525,28 +449,10 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             continue;
         case FDEFN_WIDETAG:
             adjust_pointers(where+1, 2, adj);
-            // 'raw_addr' doesn't satisfy is_lisp_pointer() for x86,
+            // For most architectures, 'raw_addr' doesn't satisfy is_lisp_pointer()
             // so adjust_pointers() would ignore it. Therefore we need to
-            // forcibly adjust it.
-#ifndef LISP_FEATURE_IMMOBILE_CODE
+            // forcibly adjust it. This is correct whether or not there are tag bits.
             adjust_word_at(where+3, adj);
-#elif defined(LISP_FEATURE_X86_64)
-            // the offset from fdefns to code can change if:
-            // - static space fdefn -> immobile space code and immobile space moved
-            // - immobile spaces changed relative position
-            {
-                // See comment above this function concerning fdefn adjustment.
-                lispobj orig = inverse_adjust(adj, (lispobj)where);
-                lispobj callee = virtual_fdefn_callee_lispobj((struct fdefn*)where, orig);
-                sword_t callee_delta = calc_adjustment(adj, callee);
-                delta = callee_delta - ((lispobj)where - orig);
-                if (delta != 0) {
-                    void* prel32 = 1 + (char*)(where+3);
-                    FIXUP_rel(UNALIGNED_STORE32(prel32, UNALIGNED_LOAD32(prel32) + delta),
-                              prel32);
-                }
-            }
-#endif
             continue;
         case CODE_HEADER_WIDETAG:
             if (filler_obj_p(where)) {
@@ -556,15 +462,21 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             // Fixup the constant pool. The word at where+1 is a fixnum.
             code = (struct code*)where;
             adjust_pointers(where+2, code_header_words(code)-2, adj);
+#if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
+            // Fixup absolute jump table
+            lispobj* jump_table = code_jumptable_start(code);
+            int count = jump_table ? *jump_table : 0;
+            int i;
+            for (i = 1; i < count; ++i) adjust_word_at(jump_table+i, adj);
+#endif
             // Fixup all embedded simple-funs
             for_each_simple_fun(i, f, code, 1, {
                 fix_fun_header_layout((lispobj*)f, adj);
 #if FUN_SELF_FIXNUM_TAGGED
-                if (f->self != (lispobj)f->code)
-                    FIXUP(f->self = (lispobj)f->code, &f->self);
-                adjust_pointers(SIMPLE_FUN_SCAV_START(f), SIMPLE_FUN_SCAV_NWORDS(f), adj);
+                if (f->self != (lispobj)f->insts)
+                    FIXUP(f->self = (lispobj)f->insts, &f->self);
 #else
-                adjust_pointers(&f->self, (lispobj*)f->code - &f->self, adj);
+                adjust_pointers(&f->self, 1, adj);
 #endif
             });
             {
@@ -587,19 +499,24 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             adjust_word_at(where+1, adj);
 #endif
             break;
-        // Vectors require extra care because of EQ-based hashing.
+        // Vectors require extra care because of address-based hashing.
         case SIMPLE_VECTOR_WIDETAG:
-          if (is_vector_subtype(*where, VectorValidHashing)) {
+          if (is_vector_subtype(*where, VectorAddrHashing)) {
               struct vector* v = (struct vector*)where;
-              gc_assert(v->length > 0 &&
-                        !(v->length & make_fixnum(1)) && // length must be even
-                        instancep(v->data[0]));
+              // If you could make a hash-table vector with space for exactly 1 k/v pair,
+              // it would have length 5.
+              gc_assert(v->length >= make_fixnum(5));
               lispobj* data = (lispobj*)v->data;
-              adjust_pointers(&data[0], 1, adj); // adjust the hash-table structure
+              adjust_pointers(&data[fixnum_value(v->length)-1], 1, adj);
+              int hwm = KV_PAIRS_HIGH_WATER_MARK(data);
               boolean needs_rehash = 0;
-              lispobj *where = &data[2], *end = &data[fixnum_value(v->length)];
+              lispobj *where = &data[2], *end = &data[2*(hwm+1)];
               // Adjust the elements, checking for need to rehash.
               for ( ; where < end ; where += 2) {
+                  // Really we should use the hash values to figure out which
+                  // keys were address-sensitive. This simply overapproximates
+                  // by assuming that any change forces rehash.
+                  // (Similar issue exists in 'fixup_space' in immobile-space.c)
                   lispobj ptr = *where; // key
                   if (is_lisp_pointer(ptr) && (delta = calc_adjustment(adj, ptr)) != 0) {
                       FIXUP(*where = ptr + delta, where);
@@ -610,7 +527,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
                       FIXUP(where[1] = ptr + delta, where+1);
               }
               if (needs_rehash) // set v->data[1], the need-to-rehash bit
-                  data[1] = make_fixnum(1);
+                  KV_PAIRS_REHASH(data) |= make_fixnum(1);
               continue;
           }
         // All the array header widetags.
@@ -690,14 +607,7 @@ static void relocate_heap(struct heap_adjust* adj)
 #else
     relocate_space(DYNAMIC_SPACE_START, (lispobj*)get_alloc_pointer(), adj);
 #endif
-
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    // Assembler routines move with varyobj space.
-    // (They're in readonly space if no immobile space)
-    struct code* code = get_asm_routine_code_component();
-    lispobj* jump_table = (lispobj*)code_text_start(code);
-    for ( ; *jump_table ; ++jump_table )
-        adjust_word_at(jump_table, adj);
     // Pointers within varyobj space to varyobj space do not need adjustment
     // so remove any delta before performing the relocation pass on this space.
     if (lisp_code_in_elf())
@@ -727,6 +637,13 @@ set_adjustment(struct heap_adjust* adj,
 #   define apply_pie_relocs(dummy1,dummy2,dummy3) (0)
 #endif
 
+/* TODO: If static + readonly were mapped as desired without disabling ASLR
+ * but one of the large spaces couldn't be mapped as desired, start over from
+ * the top, disabling ASLR. This should help to avoid relocating the heap
+ * if at all possible. It might make sense to parse the core header sooner in
+ * startup to avoid wasting time on all actions performed prior to re-exec.
+ */
+
 static void
 process_directory(int count, struct ndir_entry *entry,
                   int fd, os_vm_offset_t file_offset,
@@ -820,7 +737,7 @@ process_directory(int count, struct ndir_entry *entry,
         int compressed = id & DEFLATED_CORE_SPACE_ID_FLAG;
         id -= compressed;
         if (id < 1 || id > MAX_CORE_SPACE_ID)
-            lose("unknown space ID %ld addr %p\n", id, (void*)addr);
+            lose("unknown space ID %ld addr %p", id, (void*)addr);
 
 #ifndef LISP_FEATURE_RELOCATABLE_HEAP
         int enforce_address = 1;
@@ -851,7 +768,7 @@ process_directory(int count, struct ndir_entry *entry,
         spaces[id].base = addr;
         uword_t len = os_vm_page_size * entry->page_count;
         if (id == DYNAMIC_CORE_SPACE_ID && len > dynamic_space_size) {
-            lose("dynamic space too small for core: %luKiB required, %luKiB available.\n",
+            lose("dynamic space too small for core: %luKiB required, %luKiB available.",
                  (unsigned long)len >> 10,
                  (unsigned long)dynamic_space_size >> 10);
         }
@@ -871,7 +788,11 @@ process_directory(int count, struct ndir_entry *entry,
             else
 #endif
             if (request) {
-                addr = (uword_t)os_validate(sub_2gb_flag ? MOVABLE_LOW : MOVABLE,
+                // Avoid "Relocation of fixedobj space not supported" error by
+                // not passing relocatable flag for immobile space if text in ELF.
+                addr = (uword_t)os_validate(sub_2gb_flag ?
+                                            (lisp_code_in_elf() ? NOT_MOVABLE : MOVABLE_LOW) :
+                                            MOVABLE,
                                             (os_vm_address_t)addr, request);
                 if (!addr) {
                     lose("Can't allocate %#"OBJ_FMTX" bytes for space %ld",
@@ -959,6 +880,27 @@ process_directory(int count, struct ndir_entry *entry,
         }
     }
 
+    // Compute the bounds of the lisp assembly routines
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    asm_routines_start = VARYOBJ_SPACE_START;
+#else
+    if (widetag_of((lispobj*)READ_ONLY_SPACE_START) == CODE_HEADER_WIDETAG) {
+        asm_routines_start = READ_ONLY_SPACE_START;
+    } else {
+        lispobj *where = (lispobj*)STATIC_SPACE_START;
+        while (where < static_space_free_pointer) {
+            if (widetag_of((lispobj*)where) == CODE_HEADER_WIDETAG) {
+                asm_routines_start = (uword_t)where;
+                break;
+            }
+            where += OBJECT_SIZE(*where, where);
+        }
+        if (!asm_routines_start) lose("Can't find asm routines");
+    }
+#endif
+    asm_routines_end = asm_routines_start +
+      N_WORD_BYTES * sizetab[CODE_HEADER_WIDETAG]((lispobj*)asm_routines_start);
+
 #ifdef LISP_FEATURE_RELOCATABLE_HEAP
 #  ifdef LISP_FEATURE_GENCGC
     set_adjustment(adj, DYNAMIC_SPACE_START, // actual
@@ -1032,6 +974,8 @@ extern void gc_load_corefile_ptes(core_entry_elt_t, core_entry_elt_t,
 #define gc_load_corefile_ptes(dummy1,dummy2,dummy3,dummy4)
 #endif
 
+static void sanity_check_loaded_core(lispobj);
+
 /* 'merge_core_pages': Tri-state flag to determine whether we attempt to mark
  * pages as targets for virtual memory deduplication via MADV_MERGEABLE.
  * 1: Yes
@@ -1061,7 +1005,7 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
 
     count = read(fd, header, os_vm_page_size);
     if (count < (ssize_t) os_vm_page_size) {
-        lose("premature end of core file\n");
+        lose("premature end of core file");
     }
 
     ptr = header;
@@ -1105,6 +1049,15 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
                 // fprintf(stderr, "NOTE: TLS size increased to %x\n", dynamic_values_bytes);
             }
 #endif
+#ifdef LISP_FEATURE_GENCGC
+            if (widetag_of(native_pointer(initial_function)) == SIMPLE_FUN_WIDETAG
+                && !lisp_startup_options.noinform) {
+                fprintf(stderr, "Initial page table:\n");
+                extern void print_generation_stats(void);
+                print_generation_stats();
+            }
+#endif
+            sanity_check_loaded_core(initial_function);
             return initial_function;
         case RUNTIME_OPTIONS_MAGIC: break; // already processed
         default:
@@ -1116,13 +1069,13 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
 #include "genesis/hash-table.h"
 #include "genesis/vector.h"
 #include "genesis/cons.h"
-os_vm_address_t get_asm_routine_by_name(const char* name)
+char* get_asm_routine_by_name(const char* name)
 {
-    struct code* code = get_asm_routine_code_component();
+    struct code* code = (struct code*)asm_routines_start;
     lispobj ht = CONS(code->debug_info)->car;
     if (ht) {
         struct vector* table =
-            VECTOR(((struct hash_table*)native_pointer(ht))->table);
+            VECTOR(((struct hash_table*)native_pointer(ht))->pairs);
         lispobj sym;
         int i;
         for (i=2 ; i < fixnum_value(table->length) ; i += 2)
@@ -1139,7 +1092,224 @@ os_vm_address_t get_asm_routine_by_name(const char* name)
 
 void asm_routine_poke(const char* routine, int offset, char byte)
 {
-    char *address = (char *)get_asm_routine_by_name(routine);
+    char *address = get_asm_routine_by_name(routine);
     if (address)
         address[offset] = byte;
 }
+
+// Caution: use at your own risk
+#undef DEBUG_CORE_LOADING
+#ifdef DEBUG_CORE_LOADING
+#ifdef LISP_FEATURE_CHENEYGC
+#  error "Can't define DEBUG_CORE_LOADING for cheneygc"
+#endif
+#include "hopscotch.h"
+#include "genesis/cons.h"
+#include "genesis/layout.h"
+#include "genesis/gc-tables.h"
+#include "code.h"
+#include "gc-private.h"
+
+struct visitor {
+    // one item per value of widetag>>2
+    // element 0 is for conses, element 64 is for totals.
+    struct {
+        int count;
+        int words;
+    } headers[65], sv_subtypes[3];
+    struct hopscotch_table *reached;
+};
+
+#define RECURSE(x) if(is_lisp_pointer(x))graph_visit(ptr,x,seen)
+static void graph_visit(lispobj __attribute__((unused)) referer,
+                        lispobj ptr,
+                        struct hopscotch_table* seen)
+{
+    if (lowtag_of(ptr) == FUN_POINTER_LOWTAG
+        && widetag_of(FUNCTION(ptr)) == SIMPLE_FUN_WIDETAG)
+        ptr = fun_code_tagged(FUNCTION(ptr));
+    if (hopscotch_get(seen, ptr, 0))
+        return;
+    hopscotch_insert(seen, ptr, 1);
+    lispobj layout, bitmap, *obj;
+    int nwords, i;
+    if (lowtag_of(ptr) == LIST_POINTER_LOWTAG) {
+        RECURSE(CONS(ptr)->car);
+        RECURSE(CONS(ptr)->cdr);
+    } else switch (widetag_of(obj = native_pointer(ptr))) {
+        case SIMPLE_VECTOR_WIDETAG:
+            nwords = fixnum_value(obj[1]); // vector length
+            for(i=0; i<nwords; ++i) RECURSE(obj[i+2]);
+            break;
+        // In all the following cases except for CODE, 'nwords' is the count
+        // of payload words (following the header), so we iterate up to and
+        // including that word index. For example, if there are 2 payload words,
+        // then we scan word indices 1 and 2 off the object base address.
+        case INSTANCE_WIDETAG:
+            layout = instance_layout(obj);
+            graph_visit(ptr, layout, seen);
+            nwords = instance_length(*obj);
+            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            for(i=1; i<=nwords; ++i)
+                if (layout_bitmap_logbitp(i-1, bitmap)) RECURSE(obj[i]);
+            break;
+        case FUNCALLABLE_INSTANCE_WIDETAG:
+            layout = funinstance_layout(obj);
+            graph_visit(ptr, layout, seen);
+            bitmap = ((struct layout*)native_pointer(layout))->bitmap;
+            nwords = SHORT_BOXED_NWORDS(*obj);
+            // We don't need to scan the word at index 1 (the trampoline pointer)
+            // because it either points to the FIN itself or to readonly space.
+            for(i=2; i<=nwords; ++i)
+                if (layout_bitmap_logbitp(i-1, bitmap)) RECURSE(obj[i]);
+            break;
+        case CLOSURE_WIDETAG:
+            // We must scan the closure's trampoline word.
+            graph_visit(ptr, fun_taggedptr_from_self(obj[1]), seen);
+            // Closures can utilize one payload word beyond what the header
+            // indicates. This is quite sucky and I don't know why I did that.
+            // However, it is correctly accounted for by SHORT_BOXED_NWORDS
+            // which gives you the right number of words to scan.
+            nwords = SHORT_BOXED_NWORDS(*obj);
+            for(i=2; i<=nwords; ++i) RECURSE(obj[i]);
+            break;
+        case SYMBOL_WIDETAG:
+        case WEAK_POINTER_WIDETAG:
+            nwords = TINY_BOXED_NWORDS(*obj);
+            for(i=1; i<=nwords; ++i) RECURSE(obj[i]);
+            break;
+        case FDEFN_WIDETAG:
+            RECURSE(obj[1]);
+            RECURSE(obj[2]);
+            RECURSE(fdefn_callee_lispobj((struct fdefn*)obj));
+            break;
+        case CODE_HEADER_WIDETAG:
+            nwords = code_header_words((struct code*)obj);
+            for(i=2; i<nwords; ++i) RECURSE(obj[i]);
+            break;
+        default:
+            if (!leaf_obj_widetag_p(widetag_of(obj))) {
+                nwords = BOXED_NWORDS(*obj);
+                for(i=1; i<=nwords; ++i) RECURSE(obj[i]);
+            }
+      }
+}
+
+static void tally(lispobj ptr, struct visitor* v)
+{
+    if (lowtag_of(ptr) == LIST_POINTER_LOWTAG)
+        ++v->headers[0].count;
+    else {
+        lispobj* obj = native_pointer(ptr);
+        int widetag = widetag_of(obj);
+        int header_index = widetag>>2;
+        int words = OBJECT_SIZE(*obj, obj);
+        ++v->headers[header_index].count;
+        v->headers[header_index].words += words;
+        if (widetag == SIMPLE_VECTOR_WIDETAG) {
+            int subtype = (*obj >> N_WIDETAG_BITS) & 7;
+            if (subtype == subtype_VectorWeak)
+                subtype = 1;
+            else if (subtype & subtype_VectorHashing)
+                subtype = 2;
+            else // The above test should account for everything
+                gc_assert(subtype == 0);
+            ++v->sv_subtypes[subtype].count;
+            v->sv_subtypes[subtype].words += words;
+        }
+    }
+}
+
+static uword_t visit(lispobj* where, lispobj* limit, uword_t arg)
+{
+    struct visitor* v = (struct visitor*)arg;
+    lispobj* obj = where;
+    while (obj < limit) {
+        lispobj ptr = compute_lispobj(obj);
+        tally(ptr, v);
+        // Weird stuff happens in genesis where this logic thinks that the
+        // cold core holds a ton of unreachable objects. I have no idea why.
+        if (!hopscotch_get(v->reached, ptr, 0))
+            printf("object not reached: %p\n", (void*)ptr);
+        obj += OBJECT_SIZE(*obj, obj);
+    }
+    return 0;
+}
+
+#ifdef LISP_FEATURE_GENCGC
+#define dynamic_space_pointer_p(ptr) (find_page_index((void*)ptr) >= 0)
+#endif
+
+static void sanity_check_loaded_core(lispobj initial_function)
+{
+    struct visitor v[2];
+    struct hopscotch_table reached;
+    memset(v, 0, sizeof v);
+    // Pass 1: Count objects reachable from known roots.
+    hopscotch_create(&reached, HOPSCOTCH_HASH_FUN_DEFAULT,
+                     0, // no values
+                     1<<18, /* initial size */
+                     0);
+    {
+      lispobj* where = (lispobj*)STATIC_SPACE_START;
+      lispobj* end = static_space_free_pointer;
+      while (where<end) {
+        // This falsely treats NIL as 4 conses but it doesn't really matter.
+        // The garbage collectors do too.
+        graph_visit(0, compute_lispobj(where), &reached);
+        where += OBJECT_SIZE(*where, where);
+      }
+    }
+    graph_visit(0, initial_function, &reached); // not otherwise reachable
+    // having computed the reaching graph, tally up the dynamic space objects
+    int key_index;
+    lispobj ptr;
+    for_each_hopscotch_key(key_index, ptr, reached)
+        if (dynamic_space_pointer_p(ptr))
+            tally(ptr, &v[0]);
+    // Pass 2: Count all heap objects
+    v[1].reached = &reached;
+    walk_generation(visit, -1, (uword_t)&v[1]);
+    // Pass 3: Compare
+    // Start with the conses
+    v[0].headers[0].words = v[0].headers[0].count * 2;
+    v[1].headers[0].words = v[1].headers[0].count * 2;
+    printf("-----------------------------------------------|\n");
+    printf("       Graph walk     |         Actual         |\n");
+    printf("----------------------+------------------------|\n");
+    int i;
+    for(i=0; i<=64; ++i) {
+        // print all valid widetags (not unknown) that aren't for immediates,
+        // but always print if nonzero.
+        if (v[1].headers[i].count ||
+            ((strncmp(widetag_names[i], "unk", 3)
+              && (i != CHARACTER_WIDETAG>>2)
+              && (i != SIMPLE_FUN_WIDETAG>>2)
+              && (i != NO_TLS_VALUE_MARKER_WIDETAG>>2)
+              && (i != UNBOUND_MARKER_WIDETAG>>2)))) {
+            int mismatch = v[0].headers[i].count != v[1].headers[i].count;
+            printf("%8d %11d  | %8d %11d   | %s%s\n",
+                   v[0].headers[i].count, v[0].headers[i].words,
+                   v[1].headers[i].count, v[1].headers[i].words,
+                   i<64 ? (i ? widetag_names[i] : "cons") : "TOTAL",
+                   mismatch ? " <<<<" : "");
+            if (i == SIMPLE_VECTOR_WIDETAG>>2) {
+                int j;
+                for(j=1; j <= 2; ++j)
+                    printf("%8d %11d  | %8d %11d   |   %s\n",
+                           v[0].sv_subtypes[j].count, v[0].sv_subtypes[j].words,
+                           v[1].sv_subtypes[j].count, v[1].sv_subtypes[j].words,
+                           j==1 ? "weak" : "hashing");
+
+            }
+            v[0].headers[64].count += v[0].headers[i].count;
+            v[1].headers[64].count += v[1].headers[i].count;
+            v[0].headers[64].words += v[0].headers[i].words;
+            v[1].headers[64].words += v[1].headers[i].words;
+        }
+    }
+    hopscotch_destroy(&reached);
+}
+#else
+static void sanity_check_loaded_core(lispobj __attribute__((unused)) initial_function) {}
+#endif
